Type: Package
Package: cicalc
Title: Calculate Confidence Intervals
Version: 0.2.1
Authors@R: c(
    person("Christina", "Fillmore", , "christina.e.fillmore@gsk.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0595-2302")),
    person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd")),
    person("Mike", "Sprys", role = "aut"),
    person("Dan", "Lythgoe", role = "aut",
           comment = c(ORCID = "0000-0002-4682-772X"))
  )
Description: This calculates a variety of different CIs for proportions
    and difference of proportions that are commonly used in the
    pharmaceutical industry including Wald, Wilson, Clopper-Pearson,
    Agresti-Coull and Jeffreys for proportions. And Miettinen-Nurminen
    (1985) <doi:10.1002/sim.4780040211>, Wald, Haldane, and Mee
    <https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf> for
    difference in proportions.
License: Apache License (>= 2)
URL: https://gsk-biostatistics.github.io/cicalc/,
        https://github.com/gsk-biostatistics/cicalc
Depends: R (>= 4.1.0)
Imports: broom, cli, dplyr, forcats, glue, purrr, rlang, tidyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-08 13:44:42 UTC; christinafillmore
Author: Christina Fillmore [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0595-2302>),
  GlaxoSmithKline Research & Development Limited [cph, fnd],
  Mike Sprys [aut],
  Dan Lythgoe [aut] (ORCID: <https://orcid.org/0000-0002-4682-772X>)
Maintainer: Christina Fillmore <christina.e.fillmore@gsk.com>
Repository: CRAN
Date/Publication: 2026-05-08 14:00:08 UTC
Built: R 4.5.2; ; 2026-05-08 16:03:04 UTC; unix

Type: Package
Package: cicalc
Title: Calculate Confidence Intervals
Version: 0.2.2
Authors@R: c(
    person("Christina", "Fillmore", , "christina.e.fillmore@gsk.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0595-2302")),
    person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd")),
    person("Mike", "Sprys", role = "aut"),
    person("Dan", "Lythgoe", role = "aut",
           comment = c(ORCID = "0000-0002-4682-772X"))
  )
Description: This calculates a variety of different CIs for proportions
    and difference of proportions that are commonly used in the
    pharmaceutical industry including Wald, Wilson, Clopper-Pearson,
    Agresti-Coull and Jeffreys for proportions. And Miettinen-Nurminen
    (1985) <doi:10.1002/sim.4780040211>, Wald, Haldane, and Mee
    <https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf> for
    difference in proportions.
License: Apache License (>= 2)
URL: https://gsk-biostatistics.github.io/cicalc/,
        https://github.com/gsk-biostatistics/cicalc
Depends: R (>= 4.1.0)
Imports: broom, cli, dplyr, forcats, glue, purrr, rlang, tidyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-06-30 13:59:11 UTC; christinafillmore
Author: Christina Fillmore [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0595-2302>),
  GlaxoSmithKline Research & Development Limited [cph, fnd],
  Mike Sprys [aut],
  Dan Lythgoe [aut] (ORCID: <https://orcid.org/0000-0002-4682-772X>)
Maintainer: Christina Fillmore <christina.e.fillmore@gsk.com>
Repository: CRAN
Date/Publication: 2026-06-30 15:20:02 UTC
Built: R 4.6.1; ; 2026-06-30 16:45:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,ci_prop_diff_mh_strata)
S3method(print,ci_rel_risk_cmh_strata)
S3method(print,prop_ci_bi)
S3method(print,prop_ci_uni)
S3method(print,stratified_miettinen_nurminen)
S3method(print,stratified_wilson)
export(ci_prop_agresti_coull)
export(ci_prop_clopper_pearson)
export(ci_prop_diff_ha)
export(ci_prop_diff_haldane)
export(ci_prop_diff_jp)
export(ci_prop_diff_mee)
export(ci_prop_diff_mh_strata)
export(ci_prop_diff_mn)
export(ci_prop_diff_mn_strata)
export(ci_prop_diff_nc)
export(ci_prop_diff_nc_strata)
export(ci_prop_diff_wald)
export(ci_prop_jeffreys)
export(ci_prop_mid_p)
export(ci_prop_wald)
export(ci_prop_wilson)
export(ci_prop_wilson_strata)
export(ci_rel_risk_cmh_strata)
export(expand)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cicalc <a href='https://gsk-biostatistics.github.io/cicalc/'><img src="man/figures/cicalc.PNG" align="right" style="height:139px;"/></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/GSK-Biostatistics/cicalc/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/GSK-Biostatistics/cicalc/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/cicalc)](https://CRAN.R-project.org/package=cicalc)
[![codecov](https://codecov.io/gh/GSK-Biostatistics/cicalc/graph/badge.svg?token=jIF57l72pj)](https://app.codecov.io/gh/GSK-Biostatistics/cicalc)

<!-- badges: end -->

{cicalc} contains a number of confidence intervals commonly used in the
pharma industry. For a full list of available methods see the [package
index](https://gsk-biostatistics.github.io/cicalc/reference/index.html)

## Installation

You can install the development version of cicalc from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("GSK-Biostatistics/cicalc")
```

## Example

A subsection of common CI’s available in this package are below:

``` r
library(cicalc)
# Example of CI for proportions
x <- c(
  TRUE, TRUE, TRUE, TRUE, TRUE,
  FALSE, FALSE, FALSE, FALSE, FALSE
)
ci_prop_wald(x, conf.level = 0.9)
#> 
#> ── Wald Confidence Interval without Continuity Correction ──────────────────────
#> • 5 responses out of 10
#> • Estimate: 0.5
#> • 90% Confidence Interval:
#>   (0.2399, 0.7601)
ci_prop_wilson(x, correct = TRUE)
#> 
#> ── Wilson Confidence Interval with continuity correction ───────────────────────
#> • 5 responses out of 10
#> • Estimate: 0.5
#> • 95% Confidence Interval:
#>   (0.2366, 0.7634)
ci_prop_clopper_pearson(x)
#> 
#> ── Clopper-Pearson Confidence Interval ─────────────────────────────────────────
#> • 5 responses out of 10
#> • Estimate: 0.5
#> • 95% Confidence Interval:
#>   (0.1871, 0.8129)
ci_prop_agresti_coull(x)
#> 
#> ── Agresti-Coull Confidence Interval ───────────────────────────────────────────
#> • 5 responses out of 10
#> • Estimate: 0.5
#> • 95% Confidence Interval:
#>   (0.2366, 0.7634)
ci_prop_jeffreys(x)
#> 
#> ── Jeffreys Interval ───────────────────────────────────────────────────────────
#> • 5 responses out of 10
#> • Estimate: 0.5
#> • 95% Confidence Interval:
#>   (0.2235, 0.7765)
# Example of CI for difference of proportions 
# Generate binary samples
responses <- expand(c(9, 3), c(10, 10))
arm <- rep(c("treat", "control"), times = c(10, 10))

# Calculate 95% confidence interval for difference in proportions
ci_prop_diff_mn(x = responses, by = arm)
#> 
#> ── Miettinen-Nurminen Confidence Interval ──────────────────────────────────────
#> • 9/10 - 3/10
#> • Estimate: 0.6
#> • 95% Confidence Interval:
#>   (0.17, 0.8406)

# Stratified 
strat_df <- dplyr::tibble(
  responses = expand(c(9, 3, 7, 2), c(10, 10, 10, 10)),
  arm = rep(c("treat", "control"), 20),
  strata = rep(c("stratum1", "stratum2"), times = c(20, 20))
)

ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata, data = strat_df)
#> 
#> ── Stratified Score Miettinen-Nurminen Confidence Interval ─────────────────────
#> • 12/20 - 9/20
#> • Weights: stratum1 = 5, stratum2 = 5
#> • Estimate: 0.15
#> • 95% Confidence Interval:
#>   (-0.1606, 0.4338)
```

#' Wald CI
#'
#' Calculates the Wald interval by following the usual textbook definition
#'   for a single proportion confidence interval using the normal approximation.
#'
#' \deqn{\hat{p} \pm z_{\alpha/2} \sqrt{\frac{\hat{p}(1 - \hat{p})}{n}}}
#'
#' @param x (`binary`/`numeric`/`logical`)\cr
#'   vector of a binary values, i.e. a logical vector, or numeric with values `c(0, 1)`
#' @param conf.level (`scalar numeric`) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95
#' @param correct (`logical`)\cr apply continuity correction.
#' @param data (`data.frame`) \cr Optional data frame containing the variables specified in `x` and `by`.
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#' @export
#' @examples
#' # example code
#'x <- c(
#' TRUE, TRUE, TRUE, TRUE, TRUE,
#' FALSE, FALSE, FALSE, FALSE, FALSE
#' )
#'
#' ci_prop_wald(x, conf.level = 0.9)
#'
ci_prop_wald <- function(x, conf.level = 0.95, correct = FALSE, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_wald(
        x = x ,
        conf.level = conf.level,
        correct = correct
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)
  check_class(x = correct, "logical")
  check_scalar(correct)

  x <- stats::na.omit(x)

  n <- length(x)
  p_hat <- mean(x)
  z <- stats::qnorm((1 + conf.level) / 2)
  q_hat <- 1 - p_hat
  correction_factor <- ifelse(correct, 1 / (2 * n), 0)

  err <- z * sqrt(p_hat * q_hat) / sqrt(n) + correction_factor
  l_ci <- max(0, p_hat - err)
  u_ci <- min(1, p_hat + err)

  structure(
    list(
      N = n,
      n = sum(x),
      estimate = p_hat,
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method =
        glue::glue("Wald Confidence Interval {ifelse(correct, 'with', 'without')} Continuity Correction")
    ),
    class = c("wald", "prop_ci_uni", "cicada")
  )
}


#' Wilson CI
#'
#' Calculates the Wilson interval by calling [stats::prop.test()].
#'  Also referred to as Wilson score interval.
#'
#'
#' \deqn{\frac{\hat{p} +
#' \frac{z^2_{\alpha/2}}{2n} \pm z_{\alpha/2} \sqrt{\frac{\hat{p}(1 - \hat{p})}{n} +
#' \frac{z^2_{\alpha/2}}{4n^2}}}{1 + \frac{z^2_{\alpha/2}}{n}}}
#' @inheritParams ci_prop_wald
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#' @export
ci_prop_wilson <- function(x, conf.level = 0.95, correct = FALSE, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # check installed packages ---------------------------------------------------
  check_pkg_installed(pkg = "broom")

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_wilson(
        x = x ,
        conf.level = conf.level,
        correct = correct
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_class(x = correct, "logical")
  check_scalar(correct)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)

  x <- stats::na.omit(x)

  n <- length(x)
  y <- stats::prop.test(x = sum(x), n = n, correct = correct, conf.level = conf.level)

  structure(
    list(
      N = n,
      n = sum(x),
      conf.level = conf.level
    ) |>
      utils::modifyList(val = broom::tidy(y) |> as.list()) |>
      utils::modifyList(
        list(
          method =
            glue::glue("Wilson Confidence Interval {ifelse(correct, 'with', 'without')} continuity correction")
        )
      ),
    class = c("wilson", "prop_ci_uni", "cicada")
  )
}

#' Clopper-Pearson CI
#'
#' Calculates the Clopper-Pearson interval by calling [stats::binom.test()].
#'   Also referred to as the `exact` method.
#' @inheritParams ci_prop_wald
#'
#' @details
#' \deqn{
#' \left(1 + \frac{N-n+1}{nF[\frac{\alpha}{2};2n,2(N-n+1)]} \right)^{-1}, \left( 1 + \frac{N-n}{(n+1)F[1-\frac{\alpha}{2};2(n+1),2(N-n)]} \right)^{-1}}
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#' @export
ci_prop_clopper_pearson <- function(x, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # check installed packages ---------------------------------------------------
  check_pkg_installed(pkg = "broom")

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_clopper_pearson(
        x = x ,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }


  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)

  x <- stats::na.omit(x)
  n <- length(x)

  y <- stats::binom.test(x = sum(x), n = n, conf.level = conf.level)

  structure(
    list(N = n, n = sum(x), conf.level = conf.level) |>
      utils::modifyList(val = broom::tidy(y) |> as.list()) |>
      utils::modifyList(list(method = "Clopper-Pearson Confidence Interval")),
    class = c("clopper_pearson", "prop_ci_uni", "cicada")
  )
}

#' Agresti-Coull CI
#'
#' Calculates the `Agresti-Coull` interval (created by `Alan Agresti` and `Brent Coull`) by
#'   (for 95% CI) adding two successes and two failures to the data and then using the Wald formula to construct a CI.
#' @inheritParams ci_prop_wald
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#' @details
#' \deqn{
#' \left( \frac{\tilde{p} + z^2_{\alpha/2}/2}{n + z^2_{\alpha/2}} \pm
#' z_{\alpha/2} \sqrt{\frac{\tilde{p}(1 - \tilde{p})}{n} +
#' \frac{z^2_{\alpha/2}}{4n^2}} \right)}
#'
#' @export
ci_prop_agresti_coull <- function(x, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_agresti_coull(
        x = x ,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }
  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)

  x <- stats::na.omit(x)

  n <- length(x)
  x_sum <- sum(x)
  z <- stats::qnorm((1 + conf.level) / 2)

  # Add here both z^2 / 2 successes and failures.
  x_sum_tilde <- x_sum + z^2 / 2
  n_tilde <- n + z^2

  # Then proceed as with the Wald interval.
  p_tilde <- x_sum_tilde / n_tilde
  q_tilde <- 1 - p_tilde
  err <- z * sqrt(p_tilde * q_tilde) / sqrt(n_tilde)
  l_ci <- max(0, p_tilde - err)
  u_ci <- min(1, p_tilde + err)

  structure(
    list(
      N = n,
      n = sum(x),
      estimate = mean(x),
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method = "Agresti-Coull Confidence Interval"
    ),
    class = c("agresti_coull", "prop_ci_uni", "cicada")
  )
}

#' Jeffreys CI
#'
#' Calculates the Jeffreys interval, an equal-tailed interval based on the
#'   non-informative Jeffreys prior for a binomial proportion.
#'
#' @inheritParams ci_prop_wald
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#' @details
#' \deqn{\left( \text{Beta}\left(\frac{k}{2} + \frac{1}{2}, \frac{n - k}{2} + \frac{1}{2}\right)_\alpha,
#' \text{Beta}\left(\frac{k}{2} + \frac{1}{2}, \frac{n - k}{2} + \frac{1}{2}\right)_{1-\alpha} \right)}
#'
#' @export
ci_prop_jeffreys <- function(x, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_jeffreys(
        x = x ,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)
  x <- stats::na.omit(x)

  n <- length(x)
  x_sum <- sum(x)

  alpha <- 1 - conf.level
  l_ci <- ifelse(
    x_sum == 0,
    0,
    stats::qbeta(alpha / 2, x_sum + 0.5, n - x_sum + 0.5)
  )

  u_ci <- ifelse(
    x_sum == n,
    1,
    stats::qbeta(1 - alpha / 2, x_sum + 0.5, n - x_sum + 0.5)
  )

  structure(
    list(
      N = n,
      n = sum(x),
      estimate = mean(x),
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method = glue::glue("Jeffreys Interval")
    ),
    class = c("jeffreys", "prop_ci_uni", "cicada")
  )
}


#' Stratified Wilson CI
#'
#' Calculates the stratified Wilson confidence
#'   interval for unequal proportions as described in
#'   Xin YA, Su XG. Stratified Wilson and Newcombe confidence intervals
#'   for multiple binomial proportions. _Statistics in Biopharmaceutical Research_. 2010;2(3).
#'
#' \deqn{\frac{\hat{p}_j + \frac{z^2_{\alpha/2}}{2n_j} \pm
#' z_{\alpha/2} \sqrt{\frac{\hat{p}_j(1 - \hat{p}_j)}{n_j} +
#' \frac{z^2_{\alpha/2}}{4n_j^2}}}{1 + \frac{z^2_{\alpha/2}}{n_j}}}
#'
#' @inheritParams ci_prop_diff_mn_strata
#' @param weights (`numeric`)\cr weights for each level of the strata. If `NULL`, they are
#'   estimated using the iterative algorithm that
#'   minimizes the weighted squared length of the confidence interval.
#' @param max.iterations (positive `integer`)\cr maximum number of iterations for the iterative procedure used
#'   to find estimates of optimal weights.
#' @param correct (scalar `logical`)\cr include the continuity correction. For further information, see for example
#'   [stats::prop.test()].
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{weights}{Weights of each strata, will be the same as the input unless
#'   unspecified, then it will be the dynamically calculated weights.}
#'   \item{method}{Type of method used}
#'
#' @examples
#' # Stratified Wilson confidence interval with unequal probabilities
#'
#' set.seed(1)
#' rsp <- sample(c(TRUE, FALSE), 100, TRUE)
#' strata_data <- data.frame(
#'   "f1" = sample(c("a", "b"), 100, TRUE),
#'   "f2" = sample(c("x", "y", "z"), 100, TRUE),
#'   stringsAsFactors = TRUE
#' )
#' strata <- interaction(strata_data)
#' n_strata <- ncol(table(rsp, strata)) # Number of strata
#'
#' ci_prop_wilson_strata(
#'   x = rsp, strata = strata,
#'   conf.level = 0.90
#' )
#'
#' # Not automatic setting of weights
#' ci_prop_wilson_strata(
#'   x = rsp, strata = strata,
#'   weights = rep(1 / n_strata, n_strata),
#'   conf.level = 0.90
#' )
#'
#' @export
ci_prop_wilson_strata <- function(x,
                                  strata,
                                  weights = NULL,
                                  conf.level = 0.95,
                                  max.iterations = 10L,
                                  correct = FALSE,
                                  data = NULL) {
  set_cli_abort_call()

  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_jeffreys(
        x = x ,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_not_missing(strata)
  check_binary(x)
  check_class(correct, "logical")
  check_scalar(correct)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)
  # Change strata to be a the same length as x
  strata <- combine_strata(x, strata)
  if(!is.null(weights))
    check_identical_length(unique(strata), weights)

  if (!inherits(x, "logical")) x <- as.logical(x)
  # check all TRUE/FALSE, if so, not calculable
  if (all(x) || all(!x)) {
    cli::cli_abort("All values in {.arg x} argument are either {.code TRUE} or {.code FALSE} and CI is not estimable.")
  }



  tbl <- table(factor(x, levels = c(FALSE, TRUE)), strata, useNA = "no")
  n_strata <- length(unique(strata))

  # Checking the weights and maximum number of iterations.
  do_iter <- FALSE
  if (is.null(weights)) {
    weights <- rep(1 / n_strata, n_strata) # Initialization for iterative procedure
    do_iter <- TRUE

    # Iteration parameters
    if (!rlang::is_scalar_integerish(max.iterations) || max.iterations < 1) {
      cli::cli_abort("Argument {.arg max.iterations} must be a positive integer.")
    }
  }
  check_range(weights, range = c(0, 1), include_bounds = c(TRUE, TRUE))
  sum_weights <- sum(weights) |>
    round() |>
    as.integer()
  if (sum_weights != 1L || abs(sum_weights - sum(weights)) > sqrt(.Machine$double.eps)) {
    cli::cli_abort("The sum of the {.arg weights} argument must be {.val {1L}}")
  }

  xs <- tbl["TRUE", ]
  ns <- colSums(tbl)
  use_stratum <- (ns > 0)
  ns <- ns[use_stratum]
  xs <- xs[use_stratum]
  ests <- xs / ns
  vars <- ests * (1 - ests) / ns

  strata_qnorm <- .strata_normal_quantile(vars, weights, conf.level)

  # Iterative setting of weights if they were not passed in `weights` argument
  weights_new <- if (do_iter) {
    .update_weights_strat_wilson(vars, strata_qnorm, weights, ns, max.iterations, conf.level)$weights
  } else {
    weights
  }

  strata_conf.level <- 2 * stats::pnorm(strata_qnorm) - 1

  ci_by_strata <- Map(
    function(x, n) {
      # Classic Wilson's confidence interval
      suppressWarnings(stats::prop.test(x, n, correct = correct, conf.level = strata_conf.level)$conf.int)
    },
    x = xs,
    n = ns
  )
  lower_by_strata <- sapply(ci_by_strata, "[", 1L)
  upper_by_strata <- sapply(ci_by_strata, "[", 2L)

  lower <- sum(weights_new * lower_by_strata)
  upper <- sum(weights_new * upper_by_strata)

  # Return values
  structure(
    list(
      N = length(x),
      n = sum(x),
      estimate = mean(x),
      conf.low = lower,
      conf.high = upper,
      conf.level = conf.level,
      weights = if (do_iter) weights_new else weights,
      method =
        glue::glue("Stratified Wilson Confidence Interval {ifelse(correct, 'with', 'without')} continuity correction")
    ),
    class = c("stratified_wilson", "prop_ci_uni", "cicada")
  )
}


#' Helper Function for the Estimation of Stratified Quantiles
#'
#' This function wraps the estimation of stratified percentiles when we assume
#' the approximation for large numbers. This is necessary only in the case
#' proportions for each strata are unequal.
#'
#' @inheritParams ci_prop_wilson_strata
#'
#' @return Stratified quantile.
#'
#' @seealso [ci_prop_wilson_strata()]
#'
#' @keywords internal
#' @noRd
#'
#' @examples
#' strata_data <- table(data.frame(
#'   "f1" = sample(c(TRUE, FALSE), 100, TRUE),
#'   "f2" = sample(c("x", "y", "z"), 100, TRUE),
#'   stringsAsFactors = TRUE
#' ))
#' ns <- colSums(strata_data)
#' ests <- strata_data["TRUE", ] / ns
#' vars <- ests * (1 - ests) / ns
#' weights <- rep(1 / length(ns), length(ns))
#'
#' cicalc:::.strata_normal_quantile(vars, weights, 0.95)
.strata_normal_quantile <- function(vars, weights, conf.level) {
  summands <- weights^2 * vars
  # Stratified quantile
  sqrt(sum(summands)) / sum(sqrt(summands)) * stats::qnorm((1 + conf.level) / 2)
}

#' Helper Function for the Estimation of Weights for `ci_prop_wilson_strata()`
#'
#' This function wraps the iteration procedure that allows you to estimate
#' the weights for each proportional strata. This assumes to minimize the
#' weighted squared length of the confidence interval.
#'
#' @keywords internal
#' @inheritParams ci_prop_wilson_strata
#' @param vars (`numeric`)\cr normalized proportions for each strata.
#' @param strata_qnorm (`numeric`)\cr initial estimation with identical weights of the quantiles.
#' @param initial_weights (`numeric`)\cr initial weights used to calculate `strata_qnorm`. This can
#'   be optimized in the future if we need to estimate better initial weights.
#' @param n_per_strata (`numeric`)\cr number of elements in each strata.
#' @param max.iterations (`count`)\cr maximum number of iterations to be tried. Convergence is always checked.
#' @param tol (`number`)\cr tolerance threshold for convergence.
#'
#' @return A `list` of 3 elements: `n_it`, `weights`, and `diff_v`.
#'
#' @seealso For references and details see [`ci_prop_wilson_strata()`].
#'
#' @noRd
#' @examples
#' vs <- c(0.011, 0.013, 0.012, 0.014, 0.017, 0.018)
#' sq <- 0.674
#' ws <- rep(1 / length(vs), length(vs))
#' ns <- c(22, 18, 17, 17, 14, 12)
#'
#' cicalc:::.update_weights_strat_wilson(vs, sq, ws, ns, 100, 0.95, 0.001)
.update_weights_strat_wilson <- function(vars,
                                         strata_qnorm,
                                         initial_weights,
                                         n_per_strata,
                                         max.iterations = 50,
                                         conf.level = 0.95,
                                         tol = 0.001) {
  it <- 0
  diff_v <- NULL

  while (it < max.iterations) {
    it <- it + 1
    weights_new_t <- (1 + strata_qnorm^2 / n_per_strata)^2
    weights_new_b <- (vars + strata_qnorm^2 / (4 * n_per_strata^2))
    weights_new <- weights_new_t / weights_new_b
    weights_new <- weights_new / sum(weights_new)
    strata_qnorm <- .strata_normal_quantile(vars, weights_new, conf.level)
    diff_v <- c(diff_v, sum(abs(weights_new - initial_weights)))
    if (diff_v[length(diff_v)] < tol) break
    initial_weights <- weights_new
  }

  if (it == max.iterations) {
    warning("The heuristic to find weights did not converge with max.iterations = ", max.iterations)
  }

  list(
    "n_it" = it,
    "weights" = weights_new,
    "diff_v" = diff_v
  )
}


#' Mid-P CI
#'
#' Calculates the exact mid-p CI for binomial proportions by inverting two one-sided binomial tests that include the mid-p tail.
#' This is calculated by finding the \eqn{P_L} and \eqn{P_U} that satisfies the following equations:
#' \deqn{\sum _{x=n_1+1}^{n} \binom {n}{x} P_{L}^{x}(1-P_{L})^{n-x} + \frac{1}{2} \binom{n}{n_1} P_{L}^{n_1}(1-P_{L})^{n-n_1} = \alpha /2}
#' \deqn{\sum _{x=0}^{n_1-1} \binom {n}{x} P_{U}^{x}(1-P_{U})^{n-x} + \frac{1}{2} \binom{n}{n_1} P_{U}^{n_1}(1-P_{U})^{n-n_1} = \alpha /2}
#'
#'
#' @inheritParams ci_prop_wald
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#'
#'
#' @export
ci_prop_mid_p <- function(x, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_mid_p(
        x = x ,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)

  x <- stats::na.omit(x)

  n <- length(x)
  y <- sum(x)
  alpha <- 1 - conf.level

  if(y == 0){
    lower_ci <- 0
  } else {
    lower_ci <- stats::uniroot(alpha_test, interval=c(0.0000, 1),
                               fx=alpha_val,
                               ref_alpha = 1-alpha / 2,
                               y= y, n=n, tol=1e-08)$root
  }

  if(y == n){
    upper_ci <- 1
  } else {
    upper_ci <- stats::uniroot(alpha_test, interval=c(lower_ci, 1),
                               fx=alpha_val,
                               ref_alpha = alpha / 2,
                               y= y, n=n, tol=1e-08)$root
  }


  structure(
    list(
      N = n,
      n = sum(x),
      estimate = mean(x),
      conf.low = lower_ci,
      conf.high = upper_ci,
      conf.level = conf.level,
      method = glue::glue("Mid-p Confidence Interval")
    ),
    class = c("mid_p", "prop_ci_uni", "cicada")
  )
}

#' @keywords internal
alpha_val <- function(y, n, prob){
  0.5*stats::pbinom(y-1, n , prob, lower.tail = TRUE) +
    0.5*stats::pbinom(y, n , prob, lower.tail = TRUE)
}

#' @keywords internal
alpha_test <- function(prob, fx, ref_alpha, ...){
  alpha_prob <- fx(prob = prob, ...)
  ref_alpha - alpha_prob
}

test_that("check the ci_prop_*() functions work", {
  # setting vectors to test
  x_dbl <- c(NA, mtcars$vs)
  x_lgl <- as.numeric(x_dbl)
  x_rsp <- c(TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE)
  x_true <- rep_len(TRUE, 32)
  x_false <- rep_len(FALSE, 32)

  df <- data.frame(x_val = x_rsp)

  # check Wilson CIs ----------------------------------------------------------
  expect_snapshot(
    wilson_dbl <- ci_prop_wilson(x_dbl, conf.level = 0.9, correct = FALSE)
  )

  expect_equal(
    wilson_dbl[c("conf.low", "conf.high")],
    prop.test(x = sum(x_dbl, na.rm = TRUE), n = 32, conf.level = 0.9, correct = FALSE)$conf.int |>
      as.list() |>
      setNames(c("conf.low", "conf.high"))
  )

  expect_snapshot(
    wilsoncc_dbl <- ci_prop_wilson(x_dbl, conf.level = 0.9, correct = TRUE)
  )
  expect_equal(
    wilsoncc_dbl[c("conf.low", "conf.high")],
    prop.test(x = sum(x_dbl, na.rm = TRUE), n = 32, conf.level = 0.9, correct = TRUE)$conf.int |>
      as.list() |>
      setNames(c("conf.low", "conf.high"))
  )

  expect_snapshot(
    wilson_lgl <- ci_prop_wilson(x_lgl, conf.level = 0.9, correct = FALSE)
  )
  expect_equal(
    wilson_lgl[c("conf.low", "conf.high")],
    prop.test(x = sum(x_lgl, na.rm = TRUE), n = 32, conf.level = 0.9, correct = FALSE)$conf.int |>
      as.list() |>
      setNames(c("conf.low", "conf.high"))
  )

  wilson_dbl <- ci_prop_wilson(x_rsp, conf.level = 0.9, correct = FALSE)
  wilson_dbl_df <- ci_prop_wilson(x_val, conf.level = 0.9, correct = FALSE, data = df)
  expect_equal(wilson_dbl, wilson_dbl_df)

  expect_snapshot(ci_prop_wilson(x_rsp, conf.level = 0.9))
  expect_snapshot(ci_prop_wilson(x_true))
  expect_snapshot(ci_prop_wilson(x_false))

  # check Wald CIs ----------------------------------------------------------
  expect_snapshot(
    wald_dbl <- ci_prop_wald(x_dbl, conf.level = 0.9, correct = FALSE)
  )
  expect_snapshot(
    waldcc_dbl <- ci_prop_wald(x_dbl, conf.level = 0.9, correct = TRUE)
  )
  expect_snapshot(
    wald_lgl <- ci_prop_wald(x_lgl, conf.level = 0.9, correct = FALSE)
  )

  # Check data input
  wald_dbl <- ci_prop_wald(x_rsp, conf.level = 0.9, correct = FALSE)
  wald_dbl_df <- ci_prop_wald(x_val, conf.level = 0.9, correct = FALSE, data = df)
  expect_equal(wald_dbl, wald_dbl_df)


  expect_snapshot(ci_prop_wald(x_rsp, conf.level = 0.95, correct = TRUE))
  expect_snapshot(ci_prop_wald(x_true))
  expect_snapshot(ci_prop_wald(x_false))

  # check Clopper-Pearson CIs ----------------------------------------------------------
  expect_snapshot(
    clopper_pearson_dbl <- ci_prop_clopper_pearson(x_dbl, conf.level = 0.9)
  )
  expect_equal(
    clopper_pearson_dbl[c("conf.low", "conf.high")],
    binom.test(x = sum(x_dbl, na.rm = TRUE), n = 32, conf.level = 0.9)$conf.int |>
      as.list() |>
      setNames(c("conf.low", "conf.high"))
  )

  expect_snapshot(
    clopper_pearson_lgl <- ci_prop_clopper_pearson(x_lgl, conf.level = 0.9)
  )
  expect_equal(
    clopper_pearson_lgl[c("conf.low", "conf.high")],
    binom.test(x = sum(x_lgl, na.rm = TRUE), n = 32, conf.level = 0.9)$conf.int |>
      as.list() |>
      setNames(c("conf.low", "conf.high"))
  )

  cp_dbl <- ci_prop_clopper_pearson(x_rsp, conf.level = 0.9)
  cp_dbl_df <- ci_prop_clopper_pearson(x_val, conf.level = 0.9, data = df)
  expect_equal(cp_dbl, cp_dbl_df)

  expect_snapshot(ci_prop_clopper_pearson(x_rsp, conf.level = 0.95))
  expect_snapshot(ci_prop_clopper_pearson(x_true))
  expect_snapshot(ci_prop_clopper_pearson(x_false))

  # check Agresti-Coull CIs ----------------------------------------------------------
  expect_snapshot(
    agresti_coull_dbl <- ci_prop_agresti_coull(x_dbl, conf.level = 0.9)
  )
  expect_snapshot(
    agresti_coull_lgl <- ci_prop_agresti_coull(x_lgl, conf.level = 0.9)
  )


  ac_dbl <- ci_prop_agresti_coull(x_rsp, conf.level = 0.9)
  ac_dbl_df <- ci_prop_agresti_coull(x_val, conf.level = 0.9, data = df)
  expect_equal(ac_dbl, ac_dbl_df)


  expect_snapshot(ci_prop_agresti_coull(x_rsp, conf.level = 0.95))
  expect_snapshot(ci_prop_agresti_coull(x_true))
  expect_snapshot(ci_prop_agresti_coull(x_false))

  # check Jeffreys CIs ----------------------------------------------------------
  expect_snapshot(
    jeffreys_dbl <- ci_prop_jeffreys(x_dbl, conf.level = 0.9)
  )
  expect_snapshot(
    jeffreys_lgl <- ci_prop_jeffreys(x_lgl, conf.level = 0.9)
  )

  jeff_dbl <- ci_prop_jeffreys(x_rsp, conf.level = 0.9)
  jeff_dbl_df <- ci_prop_jeffreys(x_val, conf.level = 0.9, data = df)
  expect_equal(jeff_dbl, jeff_dbl_df)

  expect_snapshot(ci_prop_jeffreys(x_rsp, conf.level = 0.95))
  expect_snapshot(ci_prop_jeffreys(x_true))
  expect_snapshot(ci_prop_jeffreys(x_false))

  # check mid-p CIs ----------------------------------------------------------
  expect_snapshot(
    mid_p_dbl <- ci_prop_mid_p(x_dbl, conf.level = 0.9)
  )
  expect_snapshot(
    mid_p_lgl <- ci_prop_mid_p(x_lgl, conf.level = 0.9)
  )

  mid_p_dbl <- ci_prop_mid_p(x_rsp, conf.level = 0.9)
  mid_p_dbl_df <- ci_prop_mid_p(x_val, conf.level = 0.9, data = df)
  expect_equal(mid_p_dbl, mid_p_dbl_df)

  expect_snapshot(ci_prop_mid_p(x_rsp, conf.level = 0.95))
  expect_snapshot(ci_prop_mid_p(x_true))
  expect_snapshot(ci_prop_mid_p(x_false))

  # Check numbers against Agresti (2013) page 605
  expect_equal(round(ci_prop_mid_p(expand(0, 25))$conf.high, 3), 0.113)

  # https://www.lexjansen.com/wuss/2015/81_Final_Paper_PDF.pdf
  extra_test <- ci_prop_mid_p(expand(81, 263))
  expect_equal(round(extra_test$conf.low, 4), 0.2544)
  expect_equal(round(extra_test$conf.high, 4), 0.3658)

  # Test extreme values
  expect_snapshot(ci_prop_mid_p(rep(FALSE, 100)))
  expect_snapshot(ci_prop_mid_p(rep(TRUE, 100)))


  # error messaging ------------------------------------------------------------
  expect_snapshot(
    ci_prop_wilson(x_dbl, conf.level = c(0.9, 0.9)),
    error = TRUE
  )

  expect_snapshot(
    error = TRUE,
    ci_prop_wilson(mtcars$cyl)
  )
})


test_that("check the ci_prop_strat_wilson() function works", {
  # check Stratified Wilson CIs ----------------------------------------------------------
  set.seed(1)
  rsp <- c(
    sample(c(TRUE, FALSE), size = 40, prob = c(3 / 4, 1 / 4), replace = TRUE),
    sample(c(TRUE, FALSE), size = 40, prob = c(1 / 2, 1 / 2), replace = TRUE)
  )
  grp <- factor(rep(c("A", "B"), each = 40), levels = c("B", "A"))
  strata_data <- data.frame(
    "f1" = sample(c("a", "b"), 80, TRUE),
    "f2" = sample(c("x", "y", "z"), 80, TRUE),
    stringsAsFactors = TRUE
  )
  strata <- interaction(strata_data)
  weights <- 1:6 / sum(1:6)

  expect_snapshot(
    ci_prop_wilson_strata(x = rsp, strata = strata, weights = weights, correct = FALSE)
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = rsp, strata = strata, weights = weights, correct = TRUE)
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, weights = weights)
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata)
  )


  # checking error messaging
  expect_snapshot(
    ci_prop_wilson_strata(x = rep_len(TRUE, length(rsp)), strata = strata, weights = weights),
    error = TRUE
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = rep_len(FALSE, length(rsp)), strata = strata, weights = weights),
    error = TRUE
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, max.iterations = -1),
    error = TRUE
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, max.iterations = -1),
    error = TRUE
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, weights = weights + pi / 5),
    error = TRUE
  )
  expect_snapshot(
    ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, weights = weights + pi),
    error = TRUE
  )
})

#' Wald Confidence Interval for Difference in Proportions
#'
#' Calculates the Wald interval by following the usual textbook definition
#'   for a difference in proportions confidence interval using the normal approximation.
#'
#' @details
#'
#' \deqn{(\hat{p}_1 - \hat{p}_2) \pm z_{\alpha/2}
#' \sqrt{\frac{\hat{p}_1(1 - \hat{p}_1)}{n_1}+\frac{\hat{p}_2(1 - \hat{p}_2)}{n_2}}}
#'
#' @inheritParams ci_prop_diff_mn
#' @param correct (`logical`)\cr apply continuity correction.
#'
#' @return An object containing the following components:
#'   \item{n}{Number of responses in each by group}
#'   \item{N}{Total number in each by group}
#'   \item{estimate}{The point estimate of the difference in proportions (p_1 - p_2)}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Type of method used}
#'
#' @export
#'
#' @examples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_wald(x = responses, by = arm)
ci_prop_diff_wald <- function(x, by, conf.level = 0.95, correct = FALSE, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_wald(
        x = x, by = by,
        conf.level = conf.level,
        correct = correct
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)
  check_class(x = correct, "logical")
  check_scalar(correct)


  # convert vectors to count data
  df <- get_counts(x = x, by = by)

  p_1_hat <- df$response_1 / df$n_1
  p_2_hat <- df$response_2 / df$n_2
  p_hat_diff <- p_1_hat - p_2_hat

  z <- stats::qnorm((1 + conf.level) / 2)
  correction_factor <- ifelse(correct, 0.5 * (1 / df$n_1 + 1 / df$n_2), 0)

  err <- z * sqrt(p_1_hat * (1 - p_1_hat) / df$n_1 + p_2_hat * (1 - p_2_hat) / df$n_2) + correction_factor
  l_ci <- max(-1, p_hat_diff - err)
  u_ci <- min(1, p_hat_diff + err)

  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = p_hat_diff,
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method =
        glue::glue("Wald Confidence Interval {ifelse(correct, 'with', 'without')} Continuity Correction")
    ),
    class = c("wald", "prop_ci_bi", "cicada")
  )
}

#' Haldane Confidence Interval for Difference in Proportions
#'
#'
#' @inheritParams ci_prop_diff_mn
#'
#' @details
#' The confidence interval is calculated by \eqn{\theta^* \pm w} where:
#'
#' \deqn{\theta^* = \frac{(\hat{p}_1 - \hat{p}_2) + z^2v(1-2\hat{\psi})}{1+z^2u}}
#' where
#' \deqn{w = \frac{z}{1+z^2u}\sqrt{u\{4\hat{\psi}(1-\hat{\psi})-(\hat{p}_1 - \hat{p}_2)^2\}+2v(1-2\hat{\psi})(\hat{p}_1-\hat{p}_2)
#' +4z^2v^2(1-2\hat{\psi})^2
#' }}
#' \deqn{\hat{\psi} = \frac{\hat{p}_1 + \hat{p}_2}{2}}
#' \deqn{u = \frac{1/n_1 + 1/n_2}{4}}
#' \deqn{v = \frac{1/n_1 - 1/n_2}{4}}
#'
#' @return An object containing the following components:
#'
#'   \item{n}{The number of responses for each group}
#'   \item{N}{The total number in each group}
#'   \item{estimate}{The point estimate of the difference in proportions (theta*)}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Haldane Confidence Interval}
#'
#' @export
#'
#' @references
#'
#' \href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
#'
#'
#' @examples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_haldane(x = responses, by = arm)
ci_prop_diff_haldane <- function(x, by, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_haldane(
        x = x, by = by,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)


  # convert vectors to count data
  df <- get_counts(x = x, by = by)

  p_1_hat <- df$response_1 / df$n_1
  p_2_hat <- df$response_2 / df$n_2
  p_hat_diff <- p_1_hat - p_2_hat

  z <- stats::qnorm((1 + conf.level) / 2)
  psi_hat <- (p_1_hat + p_2_hat) / 2

  u <- (1 / df$n_1 + 1 / df$n_2) / 4
  v <- (1 / df$n_1 - 1 / df$n_2) / 4
  theta_star <- (p_hat_diff + z^2 * v * (1 - 2 * psi_hat)) / (1 + z^2 * u)

  a <- u * (4 * psi_hat * (1 - psi_hat) - p_hat_diff^2)
  b <- 2 * v * (1 - 2 * psi_hat) * p_hat_diff
  c <- 4 * z^2 * u^2 * (1 - psi_hat) * psi_hat
  d <- z^2 * v^2 * (1 - 2 * psi_hat)^2
  w <- z / (1 + z^2 * u) * sqrt((a + b + c + d))

  l_ci <- max(-1, theta_star - w)
  u_ci <- min(1, theta_star + w)

  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = theta_star,
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method =
        glue::glue("Haldane Confidence Interval")
    ),
    class = c("haldane", "prop_ci_bi", "cicada")
  )
}

#' Jeffreys-Perks Confidence Interval for Difference in Proportions
#'
#'
#' @inheritParams ci_prop_diff_mn
#'
#' @details
#' The confidence interval is calculated by \eqn{\theta^* \pm w} where:
#'
#' \deqn{\theta^* = \frac{(\hat{p}_1 - \hat{p}_2) + z^2v(1-2\hat{\psi})}{1+z^2u}}
#' where
#' \deqn{w = \frac{z}{1+z^2u}\sqrt{u\{4\hat{\psi}(1-\hat{\psi})-(\hat{p}_1 - \hat{p}_2)^2\}+2v(1-2\hat{\psi})(\hat{p}_1-\hat{p}_2)
#' +4z^2v^2(1-2\hat{\psi})^2
#' }}
#' \deqn{\hat{\psi} = \frac{1}{2}\left(\frac{x_1 + 1/2}{n_1+1}+\frac{x_2 + 1/2}{n_2+1}\right)}
#' \deqn{u = \frac{1/n_1 + 1/n_2}{4}}
#' \deqn{v = \frac{1/n_1 - 1/n_2}{4}}
#'
#' @return An object containing the following components:
#'
#'   \item{n}{The number of responses for each group}
#'   \item{N}{The total number in each group}
#'   \item{estimate}{The point estimate of the difference in proportions (theta*)}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Jeffreys-Perks Confidence Interval}
#'
#' @export
#'
#' @references
#'
#' \href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
#'
#'
#' @examples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_jp(x = responses, by = arm)
ci_prop_diff_jp <- function(x, by, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_jp(
        x = x, by = by,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)


  # convert vectors to count data
  df <- get_counts(x = x, by = by)

  p_1_hat <- df$response_1 / df$n_1
  p_2_hat <- df$response_2 / df$n_2
  p_hat_diff <- p_1_hat - p_2_hat

  z <- stats::qnorm((1 + conf.level) / 2)
  psi_hat <- 0.5 * ((df$response_1 + 0.5) / (df$n_1 + 1) + (df$response_2 + 0.5) / (df$n_2 + 1))

  u <- (1 / df$n_1 + 1 / df$n_2) / 4
  v <- (1 / df$n_1 - 1 / df$n_2) / 4
  theta_star <- (p_hat_diff + z^2 * v * (1 - 2 * psi_hat)) / (1 + z^2 * u)

  a <- u * (4 * psi_hat * (1 - psi_hat) - p_hat_diff^2)
  b <- 2 * v * (1 - 2 * psi_hat) * p_hat_diff
  c <- 4 * z^2 * u^2 * (1 - psi_hat) * psi_hat
  d <- z^2 * v^2 * (1 - 2 * psi_hat)^2
  w <- z / (1 + z^2 * u) * sqrt((a + b + c + d))

  l_ci <- max(-1, theta_star - w)
  u_ci <- min(1, theta_star + w)

  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = theta_star,
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method =
        glue::glue("Jeffreys-Perks Confidence Interval")
    ),
    class = c("jeffreys_perks", "prop_ci_bi", "cicada")
  )
}


#' Mee Confidence Interval for Difference in Proportions
#'
#'
#'
#' @inheritParams ci_prop_diff_mn
#'
#' @details
#' The confidence interval is calculated by \eqn{\theta^* \pm w} where:
#'
#' \deqn{\theta^* = \frac{(\hat{p}_1 - \hat{p}_2) + z^2v(1-2\hat{\psi})}{1+z^2u}}
#' where
#' \deqn{w = \frac{z}{1+z^2u}\sqrt{u\{4\hat{\psi}(1-\hat{\psi})-(\hat{p}_1 - \hat{p}_2)^2\}+2v(1-2\hat{\psi})(\hat{p}_1-\hat{p}_2)
#' +4z^2v^2(1-2\hat{\psi})^2
#' }}
#' \deqn{\hat{\psi} = \frac{1}{2}\left(\frac{x_1 + 1/2}{n_1+1}+\frac{x_2 + 1/2}{n_2+1}\right)}
#' \deqn{u = \frac{1/n_1 + 1/n_2}{4}}
#' \deqn{v = \frac{1/n_1 - 1/n_2}{4}}
#'
#' @return An object containing the following components:
#'
#'   \item{n}{The number of responses for each group}
#'   \item{N}{The total number in each group}
#'   \item{estimate}{The point estimate of the difference in proportions (p1-p2)}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Mee Confidence Interval}
#'
#' @export
#'
#' @references
#'
#' \href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
#'
#'
#' @examples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_mee(x = responses, by = arm)
ci_prop_diff_mee <- function(x, by, conf.level = 0.95, delta = NULL, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_mee(
        x = x,
        by = by,
        conf.level = conf.level,
        delta = delta
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_identical_length(x, by)
  check_numeric(delta, allow_empty = TRUE)
  check_range(delta,
    allow_empty = TRUE,
    range = c(-1, 1), include_bounds = c(FALSE, FALSE)
  )

  # convert vectors to count data
  df <- get_counts(x = x, by = by)

  alpha <- 1 - conf.level

  lower_ci <- ifelse(df$response_1 > 0,
    stats::uniroot(z_distance,
      interval = c(-0.999, 0.999),
      fx = test_score_mee,
      ref_z = stats::qnorm(1 - alpha / 2),
      s_x = df$response_1, n_x = df$n_1,
      s_y = df$response_2, n_y = df$n_2, tol = 1e-08
    )$root,
    -1
  )

  upper_ci <- ifelse(df$response_2 > 0,
    stats::uniroot(z_distance,
      interval = c(lower_ci, 0.999999),
      fx = test_score_mee,
      ref_z = stats::qnorm(alpha / 2),
      s_x = df$response_1, n_x = df$n_1,
      s_y = df$response_2, n_y = df$n_2, tol = 1e-08
    )$root,
    1
  )

  statistic <- NULL
  p.value <- NULL

  if (!is.null(delta)) {
    check_not_missing(delta)
    statistic <- test_score_mee(
      s_x = df$response_1, n_x = df$n_1,
      s_y = df$response_2, n_y = df$n_2, delta = delta
    )
    p.value <- (1 - stats::pnorm(abs(statistic)))
  }

  # Output
  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = df$response_1 / df$n_1 - df$response_2 / df$n_2,
      conf.low = lower_ci,
      conf.high = upper_ci,
      conf.level = conf.level,
      delta = delta,
      statistic = statistic,
      p.value = p.value,
      method =
        glue::glue("Mee Confidence Interval")
    ),
    class = c("mee", "prop_ci_bi", "cicada")
  )
}

#' Helper Function for the Estimation of Weights for `ci_prop_diff_mee()`
#'
#' This function is a modified mn test statistic
#'
#'
#' @keywords internal
#' @noRd
test_score_mee <- function(s_x, n_x, s_y, n_y, delta) {
  p_hat_x <- s_x / n_x
  p_hat_y <- s_y / n_y

  var_delta <- variance_mn(s_x, n_x, s_y, n_y, delta) * ((n_x + n_y - 1) / (n_x + n_y))

  T_stat <- (p_hat_x - p_hat_y - delta) / sqrt(var_delta)
  T_stat
}

#' Anderson-Hauck Confidence Interval for Difference in Proportions
#'
#'
#' @inheritParams ci_prop_diff_mn
#'
#' @details
#' The confidence interval is given by:
#'
#' \deqn{(\hat{p}_1 - \hat{p}_2) \pm \left[ \frac{1}{2 \min(n_1, n_2)} + z \sqrt{ \frac{\hat{p}_1 (1 - \hat{p}_1)}{n_1 - 1} + \frac{\hat{p}_2 (1 - \hat{p}_2)}{n_2 - 1} } \right]}.
#'
#' @return An object containing the following components:
#'
#'   \item{n}{The number of responses for each group}
#'   \item{N}{The total number in each group}
#'   \item{estimate}{The point estimate of the difference in proportions}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Anderson-Hauck Confidence Interval}
#'
#' @export
#'
#' @references
#' Hauck WW, Anderson S. (1986) A comparison of large-sample confidence interval methods for the difference of two binomial probabilities The American Statistician 40(4). p.318-322.
#' \href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
#'
#' @examples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_ha(x = responses, by = arm)
ci_prop_diff_ha <- function(x, by, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_ha(
        x = x,
        by = by,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)


  # convert vectors to count data
  df <- get_counts(x = x, by = by)

  p_1_hat <- df$response_1 / df$n_1
  p_2_hat <- df$response_2 / df$n_2
  est <- p_1_hat - p_2_hat

  z <- stats::qnorm((1 + conf.level) / 2)

  term2 <- 1 /
    (2 * min(df$n_1, df$n_2)) + z * sqrt(p_1_hat * (1 - p_1_hat) / (df$n_1 - 1) + p_2_hat * (1 - p_2_hat) / (df$n_2 - 1))

  l_ci <- max(-1, est - term2)
  u_ci <- min(1, est + term2)

  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = est,
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method =
        glue::glue("Anderson-Hauck Confidence Interval")
    ),
    class = c("Anderson-Hauck", "prop_ci_bi", "cicada")
  )
}

#' Newcombe Confidence Interval for Difference in Proportions
#'
#'
#' @inheritParams ci_prop_diff_wald
#'
#'
#' @details
#' The **Wilson (Score)** confidence limits without continuity correction for each
#' individual binomial proportion, \eqn{p_i = x_i / n_i}, for \eqn{i = 1, 2}, are given by:
#'
#' \deqn{
#' \frac{
#'   (2 n_i \hat{p}_i + z^2)
#'   \pm
#'   z \sqrt{ 4 n_i \hat{p}_i (1 - \hat{p}_i) + z^2 }
#' }{
#'   2 (n_i + z^2)
#' }
#' }
#'
#' Denote the lower and upper Wilson (Score) confidence limits for \eqn{p_i} as
#' \eqn{L_i} and \eqn{U_i}, respectively.
#'
#' Then, the **Newcombe (Score)** confidence limits for the difference in proportions
#' (\eqn{p_1 - p_2}) are given by:
#'
#' \deqn{
#' \text{Lower limit: } (\hat{p}_1 - \hat{p}_2) -
#' \sqrt{ (\hat{p}_1 - L_1)^2 + (U_2 - \hat{p}_2)^2 }
#' }
#'
#' \deqn{
#' \text{Upper limit: } (\hat{p}_1 - \hat{p}_2) +
#' \sqrt{ (U_1 - \hat{p}_1)^2 + (\hat{p}_2 - L_2)^2 }
#' }
#'
#' The confidence intervals with continuity correction for each individual binomial proportion are
#' obtained using the **Wilson (Score) confidence limits with continuity correction**.
#'
#' For each binomial proportion \eqn{p_i = x_i / n_i}, where \eqn{i = 1, 2},
#' the confidence intervals are given by:
#'
#' \deqn{
#' \frac{ 2 n_i \hat{p}_i + z^2 }{ 2 (n_i + z^2) }
#' \; \pm \;
#' \frac{ z }{ 2 (n_i + z^2) }
#' \sqrt{
#'   z^2 - \frac{2}{n_i} + 4 \hat{p}_i
#'   \left[ n_i (1 - \hat{p}_i) + 1 \right]
#' }
#' }
#'
#' @return An object containing the following components:
#'
#'   \item{n}{The number of responses for each group}
#'   \item{N}{The total number in each group}
#'   \item{estimate}{The point estimate of the difference in proportions}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{method}{Anderson-Hauck Confidence Interval}
#'
#' @export
#'
#' @references
#' Newcombe, R. G. (1998). Interval estimation for the difference between
#' independent proportions: Comparison of eleven methods. *Statistics in Medicine, 17*(8), 873–890.
#' \href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
#'
#' @examples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_nc(x = responses, by = arm)
ci_prop_diff_nc <- function(x, by, conf.level = 0.95, correct = FALSE, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_nc(
        x = x, by = by,
        conf.level = ,
        correct = correct
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_class(correct, "logical")
  check_scalar(correct)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)


  # convert vectors to count data
  df <- get_counts(x = x, by = by)


  alpha <- 1 - conf.level
  kappa <- stats::qnorm(1 - alpha / 2)
  p1_hat <- df$response_1 / df$n_1
  p2_hat <- df$response_2 / df$n_2
  est <- p1_hat - p2_hat


  if (correct) {
    w1 <- nc_binom_ci_helper(x = df$response_1, n = df$n_1, conf.level = conf.level, correct = correct)
    w2 <- nc_binom_ci_helper(x = df$response_2, n = df$n_2, conf.level = conf.level, correct = correct)
    l1 <- w1$l_ci
    u1 <- w1$u_ci
    l2 <- w2$l_ci
    u2 <- w2$u_ci
    l_ci <- max(-1, est - sqrt((p1_hat - l1)^2 + (u2 - p2_hat)^2))
    u_ci <- min(1, est + sqrt((u1 - p1_hat)^2 + (p2_hat - l2)^2))
  } else {
    w1 <- nc_binom_ci_helper(x = df$response_1, n = df$n_1, conf.level = conf.level, correct = correct)
    w2 <- nc_binom_ci_helper(x = df$response_2, n = df$n_2, conf.level = conf.level, correct = correct)
    l1 <- w1$l_ci
    u1 <- w1$u_ci
    l2 <- w2$l_ci
    u2 <- w2$u_ci
    l_ci <- est - kappa * sqrt(l1 * (1 - l1) / df$n_1 + u2 * (1 - u2) / df$n_2)
    u_ci <- est + kappa * sqrt(u1 * (1 - u1) / df$n_1 + l2 * (1 - l2) / df$n_2)
  }


  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = est,
      conf.low = l_ci,
      conf.high = u_ci,
      conf.level = conf.level,
      method =
        glue::glue("Newcombe Confidence Interval {ifelse(correct, 'with', 'without')} continuity correction")
    ),
    class = c("Newcombe", "prop_ci_bi", "cicada")
  )
}

#' Helper Function for the Estimation of Score Confidence Interval for `ci_prop_diff_nc()`
#'
#' @keywords internal
#' @noRd
nc_binom_ci_helper <- function(x, n, conf.level, correct = correct) {
  alpha <- 1 - conf.level
  kappa <- stats::qnorm(1 - alpha / 2)
  p_hat <- x / n
  q_hat <- 1 - p_hat
  est <- p_hat

  if (correct) {
    lci <- (
      2 * x + kappa^2 - 1 - kappa * sqrt(kappa^2 - 2 - 1 / n + 4 * p_hat * (n * q_hat + 1))
    ) / (2 * (n + kappa^2))
    uci <- (
      2 * x + kappa^2 + 1 + kappa * sqrt(kappa^2 + 2 - 1 / n + 4 * p_hat * (n * q_hat - 1))
    ) / (2 * (n + kappa^2))
    ci_lwr <- max(0, ifelse(p_hat == 0, 0, lci))
    ci_upr <- min(1, ifelse(p_hat == 1, 1, uci))
    list(l_ci = ci_lwr, u_ci = ci_upr)
  } else {
    term1 <- (x + kappa^2 / 2) / (n + kappa^2)
    term2 <- kappa * sqrt(n) / (n + kappa^2) * sqrt(p_hat * q_hat + kappa^2 / (4 * n))
    ci_lwr <- max(0, term1 - term2)
    ci_upr <- min(1, term1 + term2)
    list(l_ci = ci_lwr, u_ci = ci_upr)
  }
}

test_that("ci_prop_diff_* matches the values in the paper", {
  # https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf
  # 56/70-48/80
  resp <- expand(c(56, 48), c(70, 80))
  trt <- rep(c("a", "b"), times = c(70,80))

  # Wald no CC
  wald <- ci_prop_diff_wald(x = resp, by = trt)
  expect_equal(wald$conf.low, 0.0575, tolerance = 0.02)
  expect_equal(wald$conf.high, 0.3425, tolerance = 0.02)

  # Wald CC
  wald_cc <- ci_prop_diff_wald(x = resp, by = trt, correct = TRUE)
  expect_equal(wald_cc$conf.low, 0.0441, tolerance = 0.02)
  expect_equal(wald_cc$conf.high, 0.3559, tolerance = 0.02)

  # Haldane
  haldane <- ci_prop_diff_haldane(x = resp, by = trt)
  expect_equal(haldane$conf.low, 0.0535, tolerance = 0.02)
  expect_equal(haldane$conf.high, 0.3351, tolerance = 0.02)

  # Jeffreys-Perks
  jp <- ci_prop_diff_jp(x = resp, by = trt)
  expect_equal(jp$conf.low, 0.0531, tolerance = 0.02)
  expect_equal(jp$conf.high, 0.3355, tolerance = 0.02)

  # Mee
  mee <- ci_prop_diff_mee(x = resp, by = trt)
  expect_equal(mee$conf.low, 0.0533, tolerance = 0.02)
  expect_equal(mee$conf.high, 0.3377, tolerance = 0.02)

  # Anderson-Hauck
  ha <- ci_prop_diff_ha(x = resp, by = trt)
  expect_equal(ha$conf.low, 0.0494, tolerance = 0.02)
  expect_equal(ha$conf.high, 0.3506, tolerance = 0.02)

  # Newcombe (score, noCC)
  nc <- ci_prop_diff_nc(x = resp, by = trt, correct = FALSE)
  expect_equal(nc$conf.low, 0.0524, tolerance = 0.02)
  expect_equal(nc$conf.high, 0.3339, tolerance = 0.02)

  # Newcombe (score, CC)
  nc <- ci_prop_diff_nc(x = resp, by = trt, correct = TRUE)
  expect_equal(nc$conf.low, 0.0428, tolerance = 0.02)
  expect_equal(nc$conf.high, 0.3422, tolerance = 0.02)

  # 9/10-3/10
  resp <- expand(c(9, 3), c(10, 10))
  trt <- rep(c("a", "b"), times = c(10,10))

  # Wald no CC
  wald <- ci_prop_diff_wald(x = resp, by = trt)
  expect_equal(wald$conf.low, 0.2605, tolerance = 0.02)
  expect_equal(wald$conf.high, 0.9395, tolerance = 0.02)

  # Wald CC
  wald_cc <- ci_prop_diff_wald(x = resp, by = trt, correct = TRUE)
  expect_equal(wald_cc$conf.low, 0.1605, tolerance = 0.02)
  expect_equal(wald_cc$conf.high, 1.0, tolerance = 0.02)

  # Haldane
  haldane <- ci_prop_diff_haldane(x = resp, by = trt)
  expect_equal(haldane$conf.low, 0.1777, tolerance = 0.02)
  expect_equal(haldane$conf.high, 0.8289, tolerance = 0.02)

  # Jeffreys-Perks
  jp <- ci_prop_diff_jp(x = resp, by = trt)
  expect_equal(jp$conf.low, 0.1760, tolerance = 0.02)
  expect_equal(jp$conf.high, 0.8306, tolerance = 0.02)

  # Mee
  mee <- ci_prop_diff_mee(x = resp, by = trt)
  expect_equal(mee$conf.low, 0.1821, tolerance = 0.02)
  expect_equal(mee$conf.high, 0.8370, tolerance = 0.02)

  # Anderson-Hauck
  ha <- ci_prop_diff_ha(x = resp, by = trt)
  expect_equal(ha$conf.low, 0.1922, tolerance = 0.02)
  expect_equal(ha$conf.high, 1, tolerance = 0.02)

  # Newcombe (score, noCC)
  nc <- ci_prop_diff_nc(x = resp, by = trt, correct = FALSE)
  expect_equal(nc$conf.low, 0.1705, tolerance = 0.02)
  expect_equal(nc$conf.high, 0.8090, tolerance = 0.02)

  # Newcombe (score, CC)
  nc <- ci_prop_diff_nc(x = resp, by = trt, correct = TRUE)
  expect_equal(nc$conf.low, 0.1013, tolerance = 0.02)
  expect_equal(nc$conf.high, 0.8387, tolerance = 0.02)

  # 10/10 - 0/20
  resp <- expand(c(10, 0), c(10, 20))
  trt <- rep(c("a", "b"), times = c(10,20))

  # Wald no CC
  wald <- ci_prop_diff_wald(x = resp, by = trt)
  expect_equal(wald$conf.low, 1, tolerance = 0.02)
  expect_equal(wald$conf.high, 1, tolerance = 0.02)

  # Wald CC
  wald_cc <- ci_prop_diff_wald(x = resp, by = trt, correct = TRUE)
  expect_equal(wald_cc$conf.low, 0.9250, tolerance = 0.02)
  expect_equal(wald_cc$conf.high, 1, tolerance = 0.02)

  # Haldane
  haldane <- ci_prop_diff_haldane(x = resp, by = trt)
  expect_equal(haldane$conf.low, 0.7482, tolerance = 0.02)
  expect_equal(haldane$conf.high, 1.0, tolerance = 0.02)

  # Jeffreys-Perks
  jp <- ci_prop_diff_jp(x = resp, by = trt)
  expect_equal(jp$conf.low, 0.7431, tolerance = 0.02)
  expect_equal(jp$conf.high, 1, tolerance = 0.02)

  # Mee
  mee <- ci_prop_diff_mee(x = resp, by = trt)
  expect_equal(mee$conf.low, 0.7225, tolerance = 0.02)
  expect_equal(mee$conf.high, 1, tolerance = 0.02)

  # Anderson-Hauck
  ha <- ci_prop_diff_ha(x = resp, by = trt)
  expect_equal(ha$conf.low, 0.9500, tolerance = 0.02)
  expect_equal(ha$conf.high, 1, tolerance = 0.02)

  # Newcombe (score, noCC)
  nc <- ci_prop_diff_nc(x = resp, by = trt, correct = FALSE)
  expect_equal(nc$conf.low, 0.6791, tolerance = 0.02)
  expect_equal(nc$conf.high, 1, tolerance = 0.02)

  # Newcombe (score, CC)
  nc <- ci_prop_diff_nc(x = resp, by = trt, correct = TRUE)
  expect_equal(nc$conf.low, 0.6014, tolerance = 0.02)
  expect_equal(nc$conf.high, 1, tolerance = 0.02)
})

test_that("Test Data", {
  # testing with data input
  response = expand(c(56, 48), c(70, 80))
  treat = rep(c("a", "b"), times = c(70,80))
  test_df <- data.frame(response1 = response,
                      treat1 = treat
  )

  # Wald
  wald <- ci_prop_diff_wald(response, treat)
  wald_w_df <- ci_prop_diff_wald(response, treat, data = test_df)
  expect_equal(wald, wald_w_df)

  # Haldane
  haldane <- ci_prop_diff_haldane(response, treat)
  haldane_w_df <- ci_prop_diff_haldane(response, treat, data = test_df)
  expect_equal(haldane, haldane_w_df)

  # Jeffreys-Perks
  jp <- ci_prop_diff_jp(response, treat)
  jp_w_df <- ci_prop_diff_jp(response, treat, data = test_df)
  expect_equal(jp, jp_w_df)

  # Mee
  mee <- ci_prop_diff_mee(response, treat)
  mee_w_df <- ci_prop_diff_mee(response, treat, data = test_df)
  expect_equal(mee, mee_w_df)

  # Anderson-Hauck
  ha <- ci_prop_diff_ha(response, treat)
  ha_w_df <- ci_prop_diff_ha(response, treat, data = test_df)
  expect_equal(ha, ha_w_df)

  # Newcombe (no CC)
  nc <- ci_prop_diff_nc(response, treat, correct = FALSE)
  nc_w_df <- ci_prop_diff_nc(response, treat, data = test_df)
  expect_equal(nc, nc_w_df)

  # Newcombe (CC)
  nc <- ci_prop_diff_nc(response, treat, correct = TRUE)
  nc_w_df <- ci_prop_diff_nc(response, treat, correct = TRUE,data = test_df)
  expect_equal(nc, nc_w_df)

})


test_that("Test Print", {
  response = expand(c(56, 48), c(70, 80))
  treat = rep(c("a", "b"), times = c(70,80))

  # Wald no CC
  expect_snapshot(
    ci_prop_diff_wald(response, treat)
  )

  # Wald CC
  expect_snapshot(
    ci_prop_diff_wald(response, treat, correct = TRUE)
  )

  # Haldane
  expect_snapshot(
    ci_prop_diff_haldane(response, treat)
  )

  # Jeffreys-Perks
  expect_snapshot(
    ci_prop_diff_jp(response, treat)
  )

  # Mee
  expect_snapshot(
    ci_prop_diff_mee(response, treat)
  )

  # Mee with delta
  expect_snapshot(
    print(ci_prop_diff_mee(response, treat, delta = c(0, 0.1)))
  )

  # Anderson-Hauck
  expect_snapshot(
    ci_prop_diff_ha(response, treat)
  )

  # Newcombe (no CC)
  expect_snapshot(
    ci_prop_diff_nc(response, treat, correct = FALSE)
  )

  # Newcombe (CC)
  expect_snapshot(
    print(ci_prop_diff_nc(response, treat, correct = TRUE))
  )

})

#' Expand Count Data into Binary Vectors
#'
#' Converts count data (number of successes and total sample size) into a binary vector
#' of TRUE/FALSE values. This is useful for converting summary statistics back into
#' raw data format for analysis functions that require individual-level data.
#'
#' @param x Integer (or vector of integers) representing the number of successes.
#' @param n Integer (or vector of integers) representing the total number of participants.
#'
#' @return A logical vector where TRUE represents a success and FALSE represents a failure.
#'   The length of the vector equals the sum of all sample sizes.
#'
#' @details
#' For each pair of values in `x` and `n`, the function creates a vector with `x` TRUE values
#' followed by `n-x` FALSE values. If multiple pairs are provided, the resulting vectors are
#' concatenated in order.
#'
#' @examples
#' # Convert 4 successes out of 13 participants to binary data
#' expand(4, 13)
#'
#' # Convert multiple groups of data
#' # Group 1: 9 successes out of 10
#' # Group 2: 3 successes out of 10
#' expand(c(9, 3), c(10, 10))
#'
#' @export
expand <- function(x, n){
  # check inputs ---------------------------------------------------------------
  check_integerish(x)
  check_integerish(n)
  check_range(x, range = c(0, Inf), include_bounds = c(TRUE, FALSE))
  check_range(n, range = c(0, Inf), include_bounds = c(TRUE, FALSE))
  check_identical_length(x, n)
  if(any(x > n)){
    cli::cli_abort("{.arg x} must be smaller than {.arg n}")
  }

  purrr::map2(x, n, \(x1, n1){
    c(rep(TRUE, times = x1), rep(FALSE, times = n1-x1))
  }
  ) |>
    purrr::reduce(c)
}


#' To get the n's and response totals with out without strata
#' @keywords internal
#' @noMd
get_counts <- function(x, by, strata = 1) {
  dplyr::tibble(
    x = x,
    by = as.numeric(forcats::as_factor(by)),
    strata = strata
  ) |>
    dplyr::group_by(by, strata) |>
    dplyr::summarise(n = dplyr::n(),
                     response = sum(x)) |>
    tidyr::complete(strata, fill = list("n" = 0, "response" = 0)) |>
    tidyr::pivot_wider(names_from = "by", values_from = c("n", "response"))

}


#' Function to combine strata via interaction if strata is passed as a vector
#' @keywords internal
#' @noMd
combine_strata <- function(x, strata){
  if(length(strata) %% length(x) != 0){
    cli::cli_abort("The length {.arg strata} must divisable by the length {.arg x}")
  }
  factor <- length(strata) / length(x)
  split(strata, rep(1:factor, each = length(x))) |>
    interaction()
}

#' @export
print.prop_ci_uni <- function(x, ...){
  cli::cli_h1(x$method)
  cli::cli_li("{x$n} response{?s} out of {x$N}")
  cli::cli_li("Estimate: {round(x$estimate, 4)}")
  cli::cli_li("{x$conf.level*100}% Confidence Interval:")
  cli::cli_text("\u00a0\u00a0({round(x$conf.low, 4)}, {round(x$conf.high, 4)})")
}


#' @export
print.stratified_wilson <- function(x, ...){
  if(!is.null(names(x$weights))){
    name_str <- paste0(names(x$weights), " = ")
  } else {
    name_str <- ""
  }
  weight_str <- paste0(name_str, round(x$weights, 3), collapse = ", ")
  cli::cli_h1(x$method)
  cli::cli_li("{x$n} response{?s} out of {x$N}")
  cli::cli_li("Weights: {weight_str}")
  cli::cli_li("Estimate: {round(x$estimate, 3)}")
  cli::cli_li("{x$conf.level*100}% Confidence Interval:")
  cli::cli_text("\u00a0\u00a0({round(x$conf.low, 4)}, {round(x$conf.high, 4)})")
}

#' @export
print.prop_ci_bi <- function(x, ...){
  diff_str <- paste0(x$n, "/", x$N, collapse = " - ")
  cli::cli_h1(x$method)
  cli::cli_li("{diff_str}")
  cli::cli_li("Estimate: {round(x$estimate, 3)}")
  cli::cli_li("{x$conf.level*100}% Confidence Interval:")
  cli::cli_text("\u00a0\u00a0({round(x$conf.low, 4)}, {round(x$conf.high, 4)})")
  if(!is.null(x$delta)){
    cli::cli_h3("Delta")
    dplyr::tibble(d = x$delta,
                  s = x$statistic,
                  p = x$p.value) |>
      purrr::pmap(\(d,s,p){
        cli::cli_li("At {d} the statistic is {round(s, 3)} and the p-value is {round(p, 4)}")
      })
  }
  invisible(x)
}


#' @export
print.stratified_miettinen_nurminen <- function(x, ...){

  if(!is.null(names(x$weights))){
    name_str <- paste0(names(x$weights), " = ")
  } else {
    name_str <- ""
  }

  weight_str <- paste0(name_str, round(x$weights, 3), collapse = ", ")

  diff_str <- paste0(x$n, "/", x$N, collapse = " - ")
  cli::cli_h1(x$method)
  cli::cli_li("{diff_str}")
  cli::cli_li("Weights: {weight_str}")
  cli::cli_li("Estimate: {round(x$estimate, 3)}")
  cli::cli_li("{x$conf.level*100}% Confidence Interval:")
  cli::cli_text("\u00a0\u00a0({round(x$conf.low, 4)}, {round(x$conf.high, 4)})")
  if(!is.null(x$delta)){
    cli::cli_h3("Delta")
    dplyr::tibble(d = x$delta,
                  s = x$statistic,
                  p = x$p.value) |>
      purrr::pmap(\(d,s,p){
        cli::cli_li("At {d} the statistic is {round(s, 3)} and the p-value is {round(p, 4)}")
      })
  }
  invisible(x)
}


#' @export
print.ci_prop_diff_mh_strata <- function(x, ...){

  diff_str <- paste0(x$n, "/", x$N, collapse = " - ")
  cli::cli_h1(x$method)
  cli::cli_li("{diff_str}")
  cli::cli_li("Estimate: {round(x$estimate, 3)}")
  cli::cli_li("Variance: {round(x$variance, 3)}")
  cli::cli_li("{x$conf.level*100}% Confidence Interval:")
  cli::cli_text("\u00a0\u00a0({round(x$conf.low, 4)}, {round(x$conf.high, 4)})")
  invisible(x)
}

#' @export
print.ci_rel_risk_cmh_strata <- function(x, ...){
  diff_str <- paste0(x$n, "/", x$N, collapse = " - ")
  cli::cli_h1(x$method)
  cli::cli_li("{diff_str}")
  cli::cli_li("Estimate: {round(x$estimate, 3)}")
  cli::cli_li("Variance: {round(x$variance, 3)}")
  cli::cli_li("{x$conf.level*100}% Confidence Interval:")
  cli::cli_text("\u00a0\u00a0({round(x$conf.low, 4)}, {round(x$conf.high, 4)})")
  invisible(x)
}

test_that("expand converts single count to binary vector", {
  # Basic case - some successes, some failures
  expect_equal(expand(4, 10), c(rep(TRUE, 4), rep(FALSE, 6)))

  # Edge case - all successes
  expect_equal(expand(5, 5), rep(TRUE, 5))

  # Edge case - no successes
  expect_equal(expand(0, 5), rep(FALSE, 5))

  # Check that the result is a logical vector
  expect_type(expand(3, 7), "logical")
})

test_that("expand handles multiple groups correctly", {
  # Two groups with different success rates
  result <- expand(c(3, 7), c(10, 10))
  expect_equal(result, c(rep(TRUE, 3), rep(FALSE, 7), rep(TRUE, 7), rep(FALSE, 3)))
  expect_length(result, 20)

  # Three groups with different sample sizes
  result <- expand(c(2, 4, 1), c(5, 8, 3))
  expect_equal(
    result,
    c(rep(TRUE, 2), rep(FALSE, 3), rep(TRUE, 4), rep(FALSE, 4), rep(TRUE, 1), rep(FALSE, 2))
  )
  expect_length(result, 16)
})

test_that("expand handles edge cases", {

  # Zero trials
  expect_length(expand(0, 0), 0)

  # Single success in single trial
  expect_equal(expand(1, 1), TRUE)
})

test_that("expand validates inputs", {
  # Not integer-like values
  expect_error(expand("a", 5))
  expect_error(expand(5, "b"))
  expect_error(expand(1.5, 5))

  # Negative values should error
  expect_error(expand(-1, 5))
  expect_error(expand(5, -2))

  # More successes than trials should error
  expect_error(expand(6, 5))

  # Different length vectors without recycling
  expect_error(expand(c(1, 2, 3), c(5, 10)))
})

#' Mantel-Haenszel Common Risk Difference Confidence Interval
#'
#' Calculates the confidence interval for the Mantel-Haenszel estimate of the
#' common risk difference across multiple 2x2 tables (strata), using the Sato
#' or Independent Binomial variance estimator.
#'
#' The Mantel-Haenszel common risk difference is computed as:
#'
#' \deqn{\hat{\delta}_{MH} = \frac{\sum_{k} w_k \hat{\delta}_k }{\sum_{k} w_k}}
#'
#' where \eqn{w_k = \frac{n_{xk} n_{yk}}{N_k}},
#' \eqn{\hat{\delta}_k = s_{xk}/n_{xk} - y_{yk}/n_{yk}},
#' \eqn{N_k = n_{xk} + n_{yk}},
#' \eqn{s_{xk}} and \eqn{s_{yk}} are the number of events in each group, and \eqn{n_{xk}},
#' and \eqn{n_{yk}} are the group sizes in stratum \eqn{k}.
#'
#' The Sato variance is:
#'
#' \deqn{\hat{\sigma}^2(\hat{\delta}_{MH}) = \frac{\hat{d}_{MH} \sum_{k}{P_k} + \sum_k Q_k}{\left( \sum_k w_k \right)^2}}
#'
#' where \eqn{P_k = \frac{n_{xk}^2 s_{yk} - n_{yk}^2 s_{xk} + n_{xk} n_{yk} (n_{yk} - n_{xk})/2}{N_k^2}}
#' and \eqn{Q_k = \frac{s_{xk}(n_{yk} - s_{yk}) + s_{yk}(n_{xk} - s_{xk})}{2 N_k}}.
#'
#' The Cochran Independent Binomial variance is:
#'
#' \deqn{\hat{\sigma}^2(\hat{\delta}_{C}) = \sum_{k} w_k^2 \left[
#' \frac{\hat{p}_{1k}(1 - \hat{p}_{1k})}{n_{1k}} +
#' \frac{\hat{p}_{2k}(1 - \hat{p}_{2k})}{n_{2k}}
#' \right]}
#'
#' where \eqn{\hat{p}_{1k} = \frac{s_{xk}}{n_{xk}}}
#' and \eqn{\hat{p}_{2k} = \frac{s_{yk}}{n_{yk}}}.
#'
#' The confidence interval is then \eqn{\hat{\delta}_{MH} \pm z_{1-\alpha/2} \sqrt{\hat{\sigma}^2(\hat{d}_{MH})}}.
#'
#' @inheritParams ci_prop_diff_mn_strata
#' @param sato_var (`logical`)\cr Use Sato variance estimate
#'
#' @return An object containing the following components:
#'
#'   \item{estimate}{The Mantel-Haenszel estimated common risk difference}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{variance}{Variance estimate}
#'   \item{statistic}{Z-Statistic under the null hypothesis, assuming a common risk difference of 0}
#'   \item{p.value}{p-value under the null hypothesis, assuming a common risk difference of 0}
#'   \item{method}{Description of the method used ("Mantel-Haenszel Confidence Interval, Sato Variance")
#'   or ("Mantel-Haenszel Confidence Interval, Independent Binomial") }
#' @export
#' @references
#' Agresti, A. (2013). Categorical Data Analysis. 3rd Edition. John Wiley & Sons, Hoboken, NJ p. 231
#' Cochran, W.G. (1954). The Combination of estimates from different experiments. Biometrics, 10(1), p.101-129
#'
#' @examples
#' # Generate binary samples with strata
#' responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
#' arm <- rep(c("treat", "control"), 20)
#' strata <- rep(c("stratum1", "stratum2"), times = c(20, 20))
#'
#' # Calculate common risk difference
#' ci_prop_diff_mh_strata(x = responses, by = arm, strata = strata)
#' # Calculate risk difference with independent binomial variance
#' ci_prop_diff_mh_strata(x = responses, by = arm, strata = strata, sato_var = FALSE)
ci_prop_diff_mh_strata <- function(x, by, strata, conf.level = 0.95, sato_var = TRUE, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)
  if(is.data.frame(data)){
    return(
      ci_prop_diff_mh_strata(
        x = x ,
        by = by ,
        strata = strata,
        conf.level = conf.level,
        sato_var = sato_var
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_identical_length(x, by)
  check_logical(sato_var)

  strata <- combine_strata(x, strata)

  response_df <- get_counts(x = x, by = by, strata = strata)

  n_x <- response_df$n_1
  s_x <- response_df$response_1
  n_y <- response_df$n_2
  s_y <- response_df$response_2
  N_k <- n_x + n_y
  # Calculate weights and diff in weighted proportions
  weights <-(n_x * n_y) / (n_x + n_y)
  names(weights) <- response_df$strata
  tot_w <- sum(weights)

  weights_k <- weights/tot_w
  d_hat_k <- s_x/n_x - s_y/n_y

  d_mh <- sum(d_hat_k*weights_k)

  p_k <- (n_x^2*s_y - n_y^2*s_x + n_x*n_y*(n_y-n_x)/2)/N_k^2
  q_k <- (s_x*(n_y - s_y) + s_y*(n_x - s_x))/(2*N_k)

  if(sato_var) {
    est_var <- (d_mh*sum(p_k) + sum(q_k))/sum(n_x*n_y/N_k)^2
    var_title <- ", Sato Variance"
  } else {
    p1 <- s_y/n_y
    p2 <- s_x/n_x
    est_var <- sum(((p1 * (1 - p1) / n_y) + (p2 * (1 - p2) / n_x)) * weights_k^2)
    var_title <- ", Independent Binomial"
  }


  alpha <- 1 - conf.level
  z_alpha <- stats::qnorm((1 + conf.level) / 2)

  lower_ci <- d_mh - z_alpha*sqrt(est_var)
  upper_ci <- d_mh + z_alpha*sqrt(est_var)

  z_stat <- d_mh/sqrt(sato_var)

  p.value <- 2 * (1 - stats::pnorm(abs(z_stat)))

  df <- get_counts(x = x, by = by)
  # Output
  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = d_mh,
      conf.low = lower_ci,
      conf.high = upper_ci,
      conf.level = conf.level,
      variance = est_var,
      statistic = z_stat,
      p.value = p.value,
      weights = weights_k,
      method =
        glue::glue("Mantel-Haenszel Risk Difference Confidence Interval{var_title}")
    ),
    class = c("ci_prop_diff_mh_strata", "cicalc")
  )
}

#' Mantel-Haenszel Stratified Relative Risk Confidence Interval
#'
#' Calculates the confidence interval for the Mantel-Haenszel estimate of the
#' common relative risk across multiple 2x2 tables (strata)
#'
#' The Mantel-Haenszel relative risk difference is computed as:
#'
#' \deqn{RR_{MH} = \frac{\sum_{k} s_{xk}~n_{yk}/N_k}{\sum_{k} s_{yk}~n_{xk}/N_k}}
#'
#' The variance is:
#'
#' \deqn{\hat{\sigma}^2 = \hat{Var}(log(RR_{MH})) =
#' \frac{\sum_{k}(n_{xk}~n_{yk}~(s_{xk}+s_{yk}) - s_{xk}~s_{yk}~N_k)/N_k^2}
#' {(\sum_{k}s_{xk}~n_{yk}/N_k)(\sum_{k}s_{yk}~n_{xk}/N_k)}}
#'
#'
#' The confidence interval is then \eqn{\left(RR_{MH}\times exp(-z_{1-\alpha/2} \sqrt{\hat{\sigma}^2},
#' RR_{MH}\times exp(z_{1-\alpha/2} \sqrt{\hat{\sigma}^2}\right)}.
#'
#' @inheritParams ci_prop_diff_mn_strata
#'
#' @return An object containing the following components:
#'
#'   \item{estimate}{The Mantel-Haenszel estimated common risk difference}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{variance}{Mantel-Haenszel variance estimate \eqn{Var(log(RR_MH))}}
#'   \item{method}{Description of the method used ("Mantel-Haenszel Common Relative Risk Confidence Interval")}
#' @export
#' @references
#' Agresti, A. (2013). Categorical Data Analysis. 3rd Edition. John Wiley & Sons, Hoboken, NJ
#'
#' @examples
#' # Generate binary samples with strata
#' responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
#' arm <- rep(c("treat", "control"), 20)
#' strata <- rep(c("stratum1", "stratum2"), times = c(20, 20))
#'
#' # Calculate common risk difference
#' ci_rel_risk_cmh_strata(x = responses, by = arm, strata = strata)
ci_rel_risk_cmh_strata <- function(x, by, strata, conf.level = 0.95, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)
  if(is.data.frame(data)){
    return(
      ci_rel_risk_cmh_strata(
        x = x ,
        by = by ,
        strata = strata,
        conf.level = conf.level
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_identical_length(x, by)

  strata <- combine_strata(x, strata)

  response_df <- get_counts(x = x, by = by, strata = strata)

  n_x <- response_df$n_1
  s_x <- response_df$response_1
  n_y <- response_df$n_2
  s_y <- response_df$response_2
  N_k <- n_x + n_y

  rel_rik_denom <- sum(s_y*n_x/N_k)
  if(any(rel_rik_denom != 0)){
    rel_risk_mh <- sum(s_x*n_y/N_k)/rel_rik_denom
  } else {
    cli::cli_abort("Denominator of the Mantel Haenszel Estimator can not sum to 0")
  }

  var_rr <- sum((n_x*n_y*(s_x+s_y)-s_x*s_y*N_k)/N_k^2)/(sum(s_x*n_y/N_k)*sum(s_y*n_x/N_k))

  z_alpha <- stats::qnorm((1 + conf.level) / 2)

  lower_ci <- rel_risk_mh*exp(-z_alpha*sqrt(var_rr))
  upper_ci <- rel_risk_mh*exp(z_alpha*sqrt(var_rr))

  df <- get_counts(x = x, by = by)
  # Output
  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = rel_risk_mh,
      conf.low = lower_ci,
      conf.high = upper_ci,
      conf.level = conf.level,
      variance = var_rr,
      method =
        glue::glue("Mantel-Haenszel Common Relattive Risk Confidence Interval")
    ),
    class = c("ci_rel_risk_cmh_strata", "cicalc")
  )
}

#' Stratified Newcombe Common Risk Difference Confidence Interval
#'
#' Calculates the stratified Newcombe confidence interval for unequal proportions
#' as described in Yan X, Su XG. Stratified Wilson and Newcombe confidence intervals
#' or multiple binomial proportions. Weights are estimated using CMH or Wilson methods.
#'
#' \deqn{
#' L = \hat{d}_{\rm MH} - z_{\alpha/2} \sqrt{
#'   \sum_h w_h^2 L_{2h} (1 - L_{2h}) +
#'   \sum_h w_h^2 U_{1h} (1 - U_{1h})
#' }
#' }
#'
#' \deqn{
#' U = \hat{d}_{\rm MH} + z_{\alpha/2} \sqrt{
#'   \sum_h w_h^2 L_{2h} (1 - L_{2h}) +
#'   \sum_h w_h^2 U_{1h} (1 - U_{1h})
#' }
#' }
#'
#' Where:
#' - \eqn{\hat{d}_{\rm MH}}: Mantel-Haenszel common risk difference
#' - \eqn{z_{\alpha/2}}: standard normal critical value
#' - \eqn{w_h}: stratum weights
#' - \eqn{L_{2h}}, \eqn{U_{1h}}: Wilson-type CI limits for stratum h
#'
#' @inheritParams ci_prop_diff_mn_strata
#' @param weights_method (`character`)\cr Can be either "wilson" or "cmh" and directs the way weights are estimated.
#' @param correct (scalar `logical`)\cr include the continuity correction. For further information, see for example
#'   [ci_prop_diff_nc())].
#'
#' @return An object containing the following components:
#'
#'   \item{n}{Number of responses}
#'   \item{N}{Total number}
#'   \item{estimate}{The point estimate of the proportion}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{weights}{Weights of each strata calculated as per the specified "weights_method" argument.}
#'   \item{method}{Type of method used}
#'
#' @examples
#' set.seed(1)
#' rsp <- sample(c(TRUE, FALSE), 100, TRUE)
#' grp <- sample(c("Placebo", "Treatment"), 100, TRUE)
#' strata_data <- data.frame(
#'   "f1" = sample(c("a", "b"), 100, TRUE),
#'   "f2" = sample(c("x", "y", "z"), 100, TRUE),
#'   stringsAsFactors = TRUE
#' )
#' strata <- interaction(strata_data)
#'
#' ci_prop_diff_nc_strata(
#'   x = rsp, by = grp, strata = strata, weights_method = "cmh",
#'   conf.level = 0.95
#' )
#'
#' @export
ci_prop_diff_nc_strata <- function(x,
                                   by,
                                   strata,
                                   conf.level = 0.95,
                                   correct = FALSE,
                                   weights_method = c("wilson", "cmh"),
                                   data = NULL) {
  set_cli_abort_call()

  weights_method <- match.arg(weights_method)

  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_nc(
        x = x,
        by = by,
        conf.level = conf.level,
        correct = correct
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_not_missing(strata)
  check_binary(x)
  check_class(correct, "logical")
  check_scalar(correct)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_scalar(conf.level)


  if (any(tapply(x, strata, length) < 5)) {
    cli::cli_warn("Less than 5 observations in some strata.")
  }

  rsp_by_grp <- split(x, f = by)
  strata_by_grp <- split(strata, f = by)

  # Finding the weights
  weights <- if (identical(weights_method, "cmh")) {
    ci_prop_diff_mh_strata(
      x = x,
      by = by,
      strata = strata,
      conf.level = conf.level
    )$weights
  } else if (identical(weights_method, "wilson")) {
    ci_prop_wilson_strata(
      x = x,
      strata = strata,
      conf.level = conf.level,
      correct = correct
    )$weights
  }
  weights[levels(strata)[!levels(strata) %in% names(weights)]] <- 0

  # reformat method for printing
  weights_method <- if(weights_method == "cmh"){
    "CMH"
  } else{
    "Wilson"
  }

  # Calculating lower (`l`) and upper (`u`) confidence bounds per group.
  strat_wilson_by_grp <- Map(
    ci_prop_wilson_strata,
    x = rsp_by_grp,
    strata = strata_by_grp,
    conf.level = conf.level,
    correct = correct,
    weights = list(weights, weights)
  )

  l_1 <- strat_wilson_by_grp[[1]]$conf.low
  u_1 <- strat_wilson_by_grp[[1]]$conf.high
  l_2 <- strat_wilson_by_grp[[2]]$conf.low
  u_2 <- strat_wilson_by_grp[[2]]$conf.high

  # Estimating the diff and n_1, n_2 (it allows different weights to be used)
  t_tbl <- table(
    factor(x, levels = c("FALSE", "TRUE")),
    by,
    strata
  )
  n_1 <- colSums(t_tbl[1:2, 1, ])
  n_2 <- colSums(t_tbl[1:2, 2, ])
  use_stratum <- (n_1 > 0) & (n_2 > 0)
  n_1 <- n_1[use_stratum]
  n_2 <- n_2[use_stratum]
  p_1 <- t_tbl[2, 1, use_stratum] / n_1
  p_2 <- t_tbl[2, 2, use_stratum] / n_2
  est1 <- sum(weights * p_1)
  est2 <- sum(weights * p_2)
  diff_est <- est2 - est1

  lambda1 <- sum(weights^2 / n_1)
  lambda2 <- sum(weights^2 / n_2)
  z <- stats::qnorm((1 + conf.level) / 2)

  lower <- diff_est - z * sqrt(lambda2 * l_2 * (1 - l_2) + lambda1 * u_1 * (1 - u_1))
  upper <- diff_est + z * sqrt(lambda1 * l_1 * (1 - l_1) + lambda2 * u_2 * (1 - u_2))

  # Return values
  structure(
    list(
      N = length(x),
      n = sum(x),
      estimate = diff_est,
      conf.low = lower,
      conf.high = upper,
      conf.level = conf.level,
      weights = weights,
      method =
        glue::glue("Stratified Newcombe Confidence Interval {ifelse(correct, 'with', 'without')} continuity correction, {weights_method}")
    ),
    class = c("stratified_newcombe", "stratified_wilson", "cicada")
  )
}

# Data from Agresti (2013) page 226
agresti <- dplyr::tribble(
  ~centre, ~treatment, ~success, ~failure,
  1, "Drug", 11, 25,
  1, "Control", 10, 27,
  2, "Drug", 16, 4,
  2, "Control", 22, 10,
  3, "Drug", 14, 5,
  3, "Control", 7, 12,
  4, "Drug", 2, 14,
  4, "Control", 1, 16,
  5, "Drug", 6, 11,
  5, "Control", 0, 12,
  6, "Drug", 1, 10,
  6, "Control", 0, 10,
  7, "Drug", 1, 4,
  7, "Control", 1, 8,
  8, "Drug", 4, 2,
  8, "Control", 6, 1,
)
agresti_long <- agresti |>
  dplyr::mutate(
    total = success + failure,
    results = purrr::map2(success, total, \(.x, .y) expand(.x, .y)),
    centre = as.character(centre)
  ) |>
  dplyr::select(-success, -failure, -total) |>
  tidyr::unnest(results)

test_that("Testing values match what is stated in Agresti on page 231", {
  results <- ci_prop_diff_mh_strata(
    x = results, by = treatment,
    strata = centre, sato_var = TRUE, data = agresti_long
  )
  expect_equal(round(results$estimate, 3), 0.130)
  expect_equal(round(sqrt(results$variance), 3), 0.050)

  # manual calculations base on the delta and SE from agresti
  agresti_d <- 0.130
  agresti_se <- 0.05
  z <- stats::qnorm((1 + 0.95) / 2)
  low_ci <- agresti_d - agresti_se * z
  upper_ci <- agresti_d + agresti_se * z
  expect_equal(round(results$conf.low, 3), low_ci, tolerance = 0.05)
  expect_equal(round(results$conf.high, 3), upper_ci, tolerance = 0.05)
})

test_that("Check print", {
  expect_snapshot(
    ci_prop_diff_mh_strata(
      x = results, by = treatment,
      strata = centre, data = agresti_long
    )
  )
})

test_that("Testing Independent Binomial Variance Estimator", {
  results <- ci_prop_diff_mh_strata(
    x = results, by = treatment,
    strata = centre, sato_var = FALSE, data = agresti_long
  )
  results_sato <- ci_prop_diff_mh_strata(
    x = results, by = treatment,
    strata = centre, sato_var = TRUE, data = agresti_long
  )
  # estimates between the two methods should be the same
  expect_equal(results$estimate, results_sato$estimate)
  # variance between the two should not be the same
  expect_false(results$variance == results_sato$variance)

  # the results of the Independent Binomial Variance does not equal the Sato variance
  expect_false(results$conf.low == results_sato$conf.low)
  expect_false(results$conf.high == results_sato$conf.high)
})

test_that("Check print", {
  expect_snapshot(
    ci_prop_diff_mh_strata(
      x = results, by = treatment,
      strata = centre, sato_var = FALSE, data = agresti_long
    )
  )
})

# Common Relative Risk ----------------------------------------------------

test_that("Test Common Rel Risk", {
  sas_example <- dplyr::tribble(
    ~gender, ~treatment, ~response, ~count,
    "female", "Active", "Better", 16,
    "female", "Active", "Same", 11,
    "female", "Placebo", "Better", 5,
    "female", "Placebo", "Same", 20,
    "male", "Active", "Better", 12,
    "male", "Active", "Same", 16,
    "male", "Placebo", "Better", 7,
    "male", "Placebo", "Same", 19
  ) |>
    dplyr::mutate(
      res_val = response == "Better",
      res_vec = purrr::map2(res_val, count, rep)
    ) |>
    dplyr::select(-count, -res_val, -response) |>
    tidyr::unnest(res_vec)

  result <- ci_rel_risk_cmh_strata(res_vec, by = treatment, strat = gender, data = sas_example)
  expect_equal(round(result$estimate, 4), 2.1636)
  expect_equal(round(result$conf.low, 4), 1.2336)
  expect_equal(round(result$conf.high, 4), 3.7948)
})

test_that("Check print", {
  expect_snapshot(
    ci_rel_risk_cmh_strata(
      x = results, by = treatment,
      strata = centre, data = agresti_long
    )
  )
})

# Stratified Newcombe Common Risk Diffence ------------------------------------
test_that("check the ci_prop_diff_nc_strata() function works", {
  # check Stratified Newcombe CIs ----------------------------------------------------------
  set.seed(1)
  rsp <- c(
    sample(c(TRUE, FALSE), size = 40, prob = c(3 / 4, 1 / 4), replace = TRUE),
    sample(c(TRUE, FALSE), size = 40, prob = c(1 / 2, 1 / 2), replace = TRUE)
  )
  grp <- factor(rep(c("A", "B"), each = 40), levels = c("B", "A"))
  strata_data <- data.frame(
    "f1" = sample(c("a", "b"), 80, TRUE),
    "f2" = sample(c("x", "y", "z"), 80, TRUE),
    stringsAsFactors = TRUE
  )
  strata <- interaction(strata_data)

  expect_snapshot(
    ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "wilson", correct = FALSE)
  )
  expect_snapshot(
    ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "wilson", correct = TRUE)
  )

  expect_snapshot(
    ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "cmh", correct = FALSE)
  )
  expect_snapshot(
    ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "cmh", correct = TRUE)
  )

  expect_snapshot(
    ci_prop_diff_nc_strata(x = as.numeric(rsp), by = grp, strata = strata, weights_method = "wilson")
  )

  expect_snapshot(
    ci_prop_diff_nc_strata(x = as.numeric(rsp), by = grp, strata = strata, weights_method = "cmh")
  )

  expect_snapshot(
    ci_prop_diff_nc_strata(x = as.numeric(rsp), by = grp, strata = strata)
  )


  # checking error messaging
  expect_snapshot(
    ci_prop_diff_nc_strata(x = rep_len(TRUE, length(rsp)), by = grp, strata = strata, weights_method = "wilson"),
    error = TRUE
  )
  expect_snapshot(
    ci_prop_diff_nc_strata(x = rep_len(FALSE, length(rsp)), by = grp, strata = strata, weights_method = "cmh"),
    error = TRUE
  )

  # force strata with less than 5 observations, check warning
  strata[strata == "a.x"][1:11] <- "b.x"

  expect_snapshot(
    ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "wilson", correct = FALSE),
    error = TRUE
  )
})

#' Miettinen-Nurminen Confidence Interval for Difference in Proportions
#'
#' Calculates the Miettinen-Nurminen (MN) confidence interval for the difference
#' between two proportions. This method can be more accurate than traditional
#' methods, especially with small sample sizes or proportions close to 0 or 1.
#'
#' @inheritParams ci_prop_wald
#' @param by (`string`) \cr A character or factor vector with exactly two unique levels
#'   identifying the two groups to compare. Can also be a column name if a data
#'   frame provided in the `data` argument.
#' @param delta (`numeric`) \cr Optionally a single number or a vector of
#'   numbers between -1 and 1 (not inclusive) to set the difference between two
#'   groups under the null hypothesis. If provided, the function returns the
#'   test statistic and p-value under the `delta`
#'   hypothesis.
#'
#' @return An object containing the following components:
#'
#'   \item{estimate}{The point estimate of the difference in proportions (p_x - p_y)}
#'   \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used}
#'   \item{delta}{delta value(s) used}
#'   \item{statistic}{Z-Statistic under the null hypothesis based on the given 'delta'}
#'   \item{p.value}{p-value under the null hypothesis based on the given 'delta'}
#'   \item{method}{Description of the method used ("Miettinen-Nurminen Confidence Interval")}
#'
#' If `delta` is not provided statistic and p.value will be NULL
#'
#'
#' @details The function implements the Miettinen-Nurminen method to compute
#' confidence intervals for the difference between two proportions. This approach:
#' \itemize{
#' \item Calculates the Miettinen-Nurminen score test statistic for different
#' possible values of the proportion difference (delta)
#' \item Identifies the delta values where the test statistic equals the critical
#' value corresponding to the desired confidence level
#' \item Returns these boundary values as the confidence interval limits
#' }
#'
#' The method uses a score test with a small-sample correction factor, making it
#' more accurate than normal approximation methods, especially for small samples
#' or extreme proportions. The equation for the test statistics is as follows:
#'
#' \deqn{H_0: \hat{d}-\delta <= 0 \qquad \text{vs.} \qquad H_1: \hat{d}-\delta > 0}
#'
#' \deqn{ T_\delta = \frac{\hat{p_x} - \hat{p_y} - \delta}{\sigma_{mn}(\delta)}}
#'
#' where \eqn{\hat{p_*} = s_*/n_*} represent the observed number of successes
#' divided by the number of participant in that group. The \eqn{\sigma_{mn}(\delta)} is a
#' function of the delta values and is create with the following equation"
#' \eqn{\tilde{p_*}} represent the MLE of the proportions.
#' \deqn{
#' \sigma_{mn}(\delta) = \sqrt{\left[\frac{\tilde{p_y}(1-\tilde{p_y})}{n_x}+\frac{\tilde{p_x}(1-\tilde{p_x})}{n_y} \right]\left(\frac{N}{N-1}\right)}
#' }
#' \eqn{ \tilde{p_x} = 2p\cdot{cos(a)} - \frac{L_2}{3L_3}} and \eqn{ \tilde{p_y} = \tilde{p_x} + \delta}
#' where:
#' \itemize{
#'  \item \eqn{p = \pm \sqrt{\frac{L_2^2}{(3L_3)^2} - \frac{L_1}{3L_3}}}
#'  \item \eqn{a = 1/3[\pi + cos^{-1}(q/p^3)]}
#'  \item \eqn{q = \frac{L_2^3}{(3L_3)^3} - \frac{L_1L_2}{6L_3^2} + \frac{L_0}{2L_3}}
#'  \item \eqn{L_3 = n_x + n_y }
#'  \item \eqn{L_2 = (n_x + 2 n_y)\delta - N - (s_x + s_y)}
#'  \item \eqn{L_1 = (n_y\delta - L_3 - 2s_y)\delta + s_x + s_y}
#'  \item \eqn{L_0 = s_y\delta(1-\delta)}
#' }
#'
#' For more information about these equations see Miettinen (1985)
#'
#'
#' @references Miettinen, O. S., & Nurminen, M. (1985). Comparative analysis of
#' two rates. Statistics in Medicine, 4(2), 213-226.
#'
#'
#' @examples
#' # Generate binary samples
#' responses <- expand(c(9, 3), c(10, 10))
#' arm <- rep(c("treat", "control"), times = c(10, 10))
#'
#' # Calculate 95% confidence interval for difference in proportions
#' ci_prop_diff_mn(x = responses, by = arm)
#'
#' # Calculate 99% confidence interval
#' ci_prop_diff_mn(x = responses, by = arm, conf.level = 0.99)
#'
#' # Calculate the p-value under the null hypothesis delta = -0.1
#' ci_prop_diff_mn(x = responses, by = arm, delta = -0.1)
#'
#' # Calculate from a data.frame
#' data <- data.frame(responses, arm)
#' ci_prop_diff_mn(x = responses, by = arm, data = data)
#' @export
ci_prop_diff_mn <- function(x, by, conf.level = 0.95, delta = NULL, data = NULL) {
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_mn(
        x = x ,
        by = by ,
        conf.level = conf.level,
        delta = delta
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_binary(x)
  check_not_missing(by)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_identical_length(x, by)
  check_numeric(delta, allow_empty = TRUE)
  check_range(delta,allow_empty = TRUE,
              range = c(-1, 1), include_bounds = c(FALSE, FALSE))


  # convert vectors to count data
  df <- get_counts(x = x, by = by)

  alpha <- 1 - conf.level
  non_rootable <- df$response_1 == 0 | df$response_2 == 0 | df$response_1 == df$response_2
  lower_ci <- NULL
  upper_ci <- NULL
  if(!non_rootable){    
    lower_try <- purrr::safely(stats::uniroot)(z_distance, interval=c(-0.999,0.999),
                                       fx=test_score_mn,
                                       ref_z = stats::qnorm(1 - alpha / 2),
                                       s_x = df$response_1, n_x = df$n_1,
                                       s_y = df$response_2, n_y = df$n_2, tol=1e-08)
    lower_ci <- lower_try$result$root
    upper_try <- purrr::safely(stats::uniroot)(z_distance, interval=c(lower_ci,0.999),
                                       fx=test_score_mn,
                                       ref_z = stats::qnorm(alpha / 2),
                                       s_x = df$response_1, n_x = df$n_1,
                                       s_y = df$response_2, n_y = df$n_2, tol=1e-08)
    upper_ci <- upper_try$result$root
  } 
  if(non_rootable | is.null(lower_ci) | is.null(upper_ci)){
    z <- stats::qnorm((1 + conf.level) / 2)

    delta_vec <- seq(-0.99999, 0.99999, length.out = 1000000)
    T_scores <- test_score_mn(s_x = df$response_1, n_x = df$n_1,
                              s_y = df$response_2, n_y = df$n_2, delta = delta_vec)
    potential_vals <- delta_vec[which(-z < T_scores & T_scores < z)]
    lower_ci <- min(potential_vals)
    upper_ci <- max(potential_vals)
  } 
 
  statistic = NULL
  p.value = NULL

  if(!is.null(delta)){
    check_not_missing(delta)
    statistic <- test_score_mn(s_x = df$response_1, n_x = df$n_1,
                               s_y = df$response_2, n_y = df$n_2, delta = delta)
    p.value <- (1 - stats::pnorm(abs(statistic)))
  }

  # Output
  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      estimate = df$response_1/df$n_1 - df$response_2/ df$n_2,
      conf.low = lower_ci,
      conf.high = upper_ci,
      conf.level = conf.level,
      delta = delta,
      statistic = statistic,
      p.value = p.value,
      method =
        glue::glue("Miettinen-Nurminen Confidence Interval")
    ),
    class = c("miettinen-nurminen", "prop_ci_bi", "cicada")
  )

}

#' Calculate Miettinen-Nurminen Test Statistic
#'
#' This function calculates the Miettinen-Nurminen test statistic for a given
#' delta (difference in proportions) using the method described in Miettinen and
#' Nurminen (1985).
#'
#' @param s_x A numeric vector of successes in the first group.
#' @param n_x A numeric vector of sample sizes in the first group.
#' @param s_y A numeric vector of successes in the second group.
#' @param n_y A numeric vector of sample sizes in the second group.
#' @param delta A numeric value representing the hypothesized difference in proportions (p_x - p_y).
#'              Must be between -1 and 1, inclusive.
#'
#' @return A numeric value representing the Miettinen-Nurminen test statistic.
#'
#'
#' @references
#' Miettinen, O. S., & Nurminen, M. (1985). Comparative analysis of two rates.
#' Statistics in Medicine, 4(2), 213-226.
#'
#'
#' @keywords internal
#' @noRd
test_score_mn <- function(s_x, n_x, s_y, n_y, delta){
  p_hat_x <- s_x / n_x
  p_hat_y <- s_y / n_y

  var_delta <- variance_mn(s_x, n_x, s_y, n_y, delta)

  T_stat <- (p_hat_x - p_hat_y - delta) / sqrt(var_delta)
  T_stat
}


#' Stratified Miettinen-Nurminen Confidence Interval for Difference in Proportions
#'
#' Calculates Stratified Miettinen-Nurminen (MN) confidence intervals and
#' corresponding point estimates for the difference between two proportions
#'
#' @inheritParams ci_prop_diff_mn
#' @param strata (`numeric`) \cr A vector specifying the stratum for each observation. It needs
#'   to be the length of x or a multiple of x if multiple levels of strata are
#'   present. Can also be a column name (or vector of column names NOT quoted)
#'   if a data frame provided in the `data` argument.
#' @param method (`string`) \cr Specifying how the CIs should be calculated. It must
#'   equal either 'score' or 'summary score'. See details for more information
#'   about the implementation differences.
#'
#'
#' @return An object containing the following components:
#'
#'   \item{estimate}{The point estimate of the difference in proportions (p_x -
#'   p_y)} \item{conf.low}{Lower bound of the confidence interval}
#'   \item{conf.high}{Upper bound of the confidence interval}
#'   \item{conf.level}{The confidence level used} \item{delta}{delta value(s) used}
#'   \item{statistic}{Z-Statistic under the null hypothesis based on the given 'delta'}
#'   \item{p.value}{p-value under the null hypothesis based on the given 'delta'}
#'   \item{method}{Description of the method used ("Stratified \{method\}
#'   Miettinen-Nurminen Confidence Interval")}
#'
#'   If `delta` is not provided statistic and p.value will be NULL
#'
#'
#' @details The function implements the stratified Miettinen-Nurminen method to compute
#'   confidence intervals for the difference between two proportions across multiple strata.
#'   \deqn{H_0: \hat{d}-\delta <= 0 \qquad \text{vs.} \qquad H_1: \hat{d}-\delta > 0}
#'
#'   The "score" method is a weighted MN score first described in the original 1985
#'   paper. The formula is:
#'   \itemize{
#'     \item Calculates weights for each stratum as \eqn{w_i = \frac{n_{xi} \cdot n_{yi}}{n_{xi} + n_{yi}}}
#'     \item Computes the overall weighted difference \eqn{\hat{d} = \frac{\sum w_i \hat{p}_{xi}}{\sum w_i} -
#'           \frac{\sum w_i \hat{p}_{yi}}{\sum w_i}}
#'     \item Uses the stratified test statistic: \deqn{Z_{\delta} = \frac{\hat{d} - \delta}
#'           {\sqrt{\sum_{i=1}^k \left(\frac{w_i}{\sum w_i}\right)^2 \cdot \hat{\sigma}_{mn}^2({d})}}}
#'     \item Finds the range of all values of \eqn{\delta} for which the stratified test statistic (\eqn{Z_\delta})
#'           falls in the acceptance region \eqn{\{ Z_\delta < z_{\alpha/2}\}}
#'   }
#'
#'   The \eqn{\hat{\sigma}_{mn}^2(\hat{d})} is the Miettinen-Nurminen variance estimate.
#'   See the details of [ci_prop_diff_mn()] for how \eqn{\hat{\sigma}_{mn}^2(\delta)} is calculated.
#'
#'   The "summary score" method follows the meta-analyses proposed in Agresti
#'   2013 and is consistent with the "Summary Score Confidence Limits" method
#'   used in SAS. The formula is:
#'   \itemize{
#'   \item The point estimate of the stratified risk difference is a weighted average of the midpoints of the within-stratum MN confidence intervals:
#'     \deqn{
#'       \hat{d}_{\text{S}} = \sum_i \hat{d}_i w_i
#'       }
#'   \item Define \eqn{s_i} as the width of the CI for the \eqn{i}th stratum divided by \eqn{2 \times z_{\alpha/2}} and then stratum weights are given by
#'   \deqn{
#'     w_i = \left( \frac{1}{s_i^2} \right) \bigg/ \sum_i \left( \frac{1}{s_i^2} \right)
#'     }
#'   \item The variance of \eqn{\hat{d}_{\text{S}} } is computed as
#'   \deqn{
#'      \widehat{\text{Var}}(\hat{d}_{\text{S}}) = \frac{1}{\sum_i \left( \frac{1}{s_i^2} \right) }
#'     }
#'   \item Confidence limits for the stratified risk difference estimate are
#'   \deqn{
#'     \hat{d}_{\text{S}} \pm \left( z_{\alpha /2} \times \widehat{\text{Var}}(\hat{d}_{\text{S}}) \right)
#'     }
#'   }
#'
#'
#' @references
#' Miettinen, O. S., & Nurminen, M. (1985). Comparative analysis of two rates.
#' Statistics in Medicine, 4(2), 213-226.
#'
#' \href{https://documentation.sas.com/doc/en/pgmsascdc/9.4_3.5/procstat/procstat_freq_details63.htm}{Common Risk Difference :: Base SAS(R) 9.4 Procedures Guide: Statistical Procedures, Third Edition}
#'
#' Agresti, A. (2013). Categorical Data Analysis. 3rd Edition. John Wiley & Sons, Hoboken, NJ
#'
#' @examples
#' # Generate binary samples with strata
#' responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
#' arm <- rep(c("treat", "control"), 20)
#' strata <- rep(c("stratum1", "stratum2"), times = c(20, 20))
#'
#' # Calculate stratified confidence interval for difference in proportions
#' ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata)
#'
#' # Using the summary score method
#' ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata,
#'                       method = "summary score")
#'
#' # Calculate 99% confidence interval
#' ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata,
#'                       conf.level = 0.99)
#'
#' # Calculate p-value under null hypothesis delta = 0.2
#' ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata,
#'                       delta = 0.2)
#'
#' @export
ci_prop_diff_mn_strata <- function(x, by, strata, method = c("score", "summary score"),
                                   conf.level = 0.95, delta = NULL, data = NULL){
  set_cli_abort_call()
  check_data_frame(data, allow_empty = TRUE)
  if(is.data.frame(data)){
    return(
      ci_prop_diff_mn_strata(
        x = x ,
        by = by ,
        strata = strata,
        method = method,
        conf.level = conf.level,
        delta = delta
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  # check inputs ---------------------------------------------------------------
  check_not_missing(x)
  check_not_missing(by)
  check_not_missing(strata)
  check_binary(x)
  check_n_levels(by, n_levels = 2)
  check_range(conf.level, range = c(0, 1), include_bounds = c(FALSE, FALSE))
  check_identical_length(x, by)
  check_numeric(delta, allow_empty = TRUE)
  check_range(delta,allow_empty = TRUE,
              range = c(-1, 1), include_bounds = c(FALSE, FALSE))

  method <- match.arg(method, c("score", "summary score"))
  alpha <- 1 - conf.level

  # if data was passed, evaluate in the context of the data frame
  if (is.data.frame(data)) {
    return(
      ci_prop_diff_mn_strata(
        x = x ,
        by = by ,
        strata = strata,
        conf.level = conf.level,
        delta = delta
      ) |>
        substitute() |>
        eval(envir = data, enclos = parent.frame())
    )
  }

  strata <- combine_strata(x, strata)

  statistic <- NULL
  p.value <- NULL
  if(method == "score"){
    # Get the n and response for each by and strata
    by_strata <- interaction(by, strata) |>
      unique()
    if(length(by_strata) == 2){
      cli::cli_abort("{.arg by} and {.arg strata} are non-unique check inputs")
    }
    response_df <- get_counts(x = x, by = by, strata = strata)
    n_x <- response_df$n_1
    s_x <- response_df$response_1
    n_y <- response_df$n_2
    s_y <- response_df$response_2

    if(0 %in% c(s_x, s_y)){
        cli::cli_text("At least one stratum has a 0 response")
      }
    # Calculate weights and diff in weighted proportions
    weights <-(n_x * n_y) / (n_x + n_y)
    names(weights) <- response_df$strata
    tot_w <- sum(weights, na.rm = TRUE)
    diff <- sum(s_x/n_x*weights, na.rm = TRUE)/tot_w - sum(s_y/n_y*weights, na.rm = TRUE)/tot_w

    # Calculate confidence interval
    lower_ci <- stats::uniroot(z_distance, interval=c(-0.999,0.999),
                                      fx=test_score_mn_weighted,
                                      ref_z = stats::qnorm(1 - alpha / 2),
                                      s_x = s_x, n_x = n_x,
                                      s_y = s_y, n_y = n_y, w = weights, tol=1e-08)$root

    upper_ci <- stats::uniroot(z_distance, interval=c(-0.999,0.999),
                                      fx=test_score_mn_weighted,
                                      ref_z = stats::qnorm(alpha / 2),
                                      s_x = s_x, n_x = n_x,
                                      s_y = s_y, n_y = n_y, w = weights, tol=1e-08)$root

    if(!is.null(delta)){
      statistic <- purrr::map_dbl(delta, \(d){
        test_score_mn_weighted(s_x = s_x, n_x = n_x,
                               s_y = s_y, n_y = n_y, w = weights, delta = d)
      })
      p.value <- (1 - stats::pnorm(abs(statistic)))
    }
  } else if(method == "summary score") {

    #SAS PROC FREQ Summary Score Estimate of the Common Risk Difference
    #https://support.sas.com/documentation/cdl/en/procstat/67528/HTML/default/viewer.htm#procstat_freq_details63.htm
    
    estimate_df <- dplyr::tibble(
      x = x,
      by = as.numeric(as.factor(by)),
      strata = strata
    ) |>
      dplyr::group_by(strata) |>
      dplyr::mutate(n_in_strata = dplyr::n()) |>
      dplyr::filter(.data$n_in_strata > 1) |>
      dplyr::summarise(mn = list(ci_prop_diff_mn(x, by, conf.level =conf.level))) |>
      dplyr::mutate(
        low = purrr::map_dbl(.data$mn, "conf.low"),
        high = purrr::map_dbl(.data$mn, "conf.high"),
        width = .data$high - .data$low,
        dh = .data$low + .data$width/2,
        sh = .data$width/(2*stats::qnorm(1-alpha/2)),
        w = (1/.data$sh^2)/sum(1/.data$sh^2)
      )
    weights <- estimate_df$w
    names(weights) <- estimate_df$strata

    estimate <- estimate_df |>
      dplyr::summarise(dS = sum(.data$dh*.data$w),
                       var_ds = 1/sum(1/.data$sh^2))


    # Calculate confidence interval
    lower_ci <- estimate$dS - stats::qnorm(1-alpha/2)*sqrt(estimate$var_ds)
    upper_ci <- estimate$dS + stats::qnorm(1-alpha/2)*sqrt(estimate$var_ds)
    diff <- estimate$dS
    if(!is.null(delta)){
      response_df <- get_counts(x = x, by = by, strata = strata)
      n_x <- response_df$n_1
      s_x <- response_df$response_1

      n_y <- response_df$n_2
      s_y <- response_df$response_2

      if(0 %in% c(s_x, s_y)){
        cli::cli_text("At least one stratum has a 0 response")
      }
      # Calculate weights and diff in weighted proportions
      w <-(n_x * n_y) / (n_x + n_y)

      statistic <- test_score_mn_weighted(s_x = s_x, n_x = n_x,
                                          s_y = s_y, n_y = n_y, w = w, delta = delta)
      p.value <- (1 - stats::pnorm(abs(statistic)))
    }

  }
  df <- get_counts(x = x, by = by)

  structure(
    list(
      n = c(df$response_1, df$response_2),
      N = c(df$n_1, df$n_2),
      weights = weights,
      estimate = diff,
      conf.low = lower_ci,
      conf.high = upper_ci,
      conf.level = conf.level,
      delta = delta,
      statistic = statistic,
      p.value = p.value,
      method =
        glue::glue("Stratified {stringr::str_to_title(method)} Miettinen-Nurminen Confidence Interval")
    ),
    class = c("stratified_miettinen_nurminen", "prop_ci_bi", "cicada")
  )

}

#' Calculate Stratified Miettinen-Nurminen Test Statistic
#'
#' The stratified Miettinen-Nurminen z test statistic
#' for a given delta (difference in proportions) using the method described in
#' Miettinen and Nurminen (1985), adapted for stratified data.
#'
#' @param s_x A numeric vector of successes in the first group, one per stratum.
#' @param n_x A numeric vector of sample sizes in the first group, one per stratum.
#' @param s_y A numeric vector of successes in the second group, one per stratum.
#' @param n_y A numeric vector of sample sizes in the second group, one per stratum.
#' @param delta A numeric value representing the hypothesized difference in proportions (p_x - p_y).
#'        Must be between -1 and 1, inclusive.
#'
#' @return A numeric value representing the stratified Miettinen-Nurminen Z-test statistic.
#'
#' @details
#' The function implements the stratified version of the Miettinen-Nurminen (MN) score test
#' for the difference between two proportions. The stratified test combines information across
#' multiple strata using appropriate weighting.
#'
#'
#' @references
#' Miettinen, O. S., & Nurminen, M. (1985). Comparative analysis of two rates.
#' Statistics in Medicine, 4(2), 213-226.
#'
#' @keywords internal
#' @noRd
test_score_mn_weighted<-function(s_x, n_x, s_y, n_y, w, delta){
  tot_w <- sum(w, na.rm = TRUE)
  diff <- sum(s_x/n_x*w, na.rm = TRUE)/tot_w - sum(s_y/n_y*w, na.rm = TRUE)/tot_w

  mV <- variance_mn(s_x, n_x, s_y, n_y, delta)

  #equation 15
  den <- ((w/tot_w)^2)*mV
  tot_den <- sum(den, na.rm = TRUE)

  zstat <- (diff-delta)/sqrt(tot_den)
  zstat
}



#' @keywords internal
#'
variance_mn <- function(s_x, n_x, s_y, n_y, delta){
  # The implementation follows equations described in Miettinen and Nurminen's
  # paper, specifically equations 8, 15, 27, and 28 for handling stratified
  # data. Variable names have been changed to be more consistent with the rest
  # of the package
  N <- n_x + n_y
  tot_s <- s_x + s_y

  #equation 27
  L3 <- N
  L2 <- (n_x + 2*n_y)*delta - N - tot_s
  L1 <- (n_y*delta - N - 2*s_y)*delta + tot_s
  L0 <- s_y*delta*(1-delta)

  #equation 28
  q <- (L2^3)/((3*L3)^3) - (L1*L2)/(6*(L3^2)) + L0/(2*L3)

  p <- sign(q)*sqrt((L2^2)/((3*L3)^2) - L1/(3*L3))

  temp <- pmax(pmin(q/(p^3),1),-1)
  a <- (1/3)*(pi+acos(temp))

  # The MLE of p_x and p_y
  mle_p_x <- 2*p*cos(a) - L2/(3*L3)
  mle_p_y <-  mle_p_x+delta

  #equation 8
  var_delta <- (mle_p_y*(1-mle_p_y)/n_x + mle_p_x*(1-mle_p_x)/n_y) * (N/(N-1))
  var_delta
}


#' @keywords internal
z_distance <- function(delta, fx, ref_z, ...){
  z_delta <- fx(delta = delta, ...)
  ref_z - z_delta
}

test_that("ci_prop_diff_mn matches the values in the paper", {
  # https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf
  # 56/70-48/80
  resp <- expand(c(56, 48), c(70, 80))
  trt <- rep(c("a", "b"), times = c(70,80))
  norm <- ci_prop_diff_mn(x = resp, by = trt)

  expect_equal(norm$conf.low, 0.0528, tolerance = 0.02)
  expect_equal(norm$conf.high, 0.3382, tolerance = 0.02)


  # 9/10-3/10
  resp <- expand(c(9, 3), c(10, 10))
  trt <- rep(c("a", "b"), times = c(10,10))
  small <- ci_prop_diff_mn(x = resp, by = trt)

  expect_equal(small$conf.low, 0.1700, tolerance = 0.3)
  expect_equal(small$conf.high, 0.8406, tolerance = 0.02)


  # 10/10 - 0/20
  resp <- expand(c(10, 0), c(10, 20))
  trt <- rep(c("a", "b"), times = c(10,20))
  extreme <- ci_prop_diff_mn(x = resp, by = trt)

  expect_equal(extreme$conf.low, 0.7156, tolerance = 0.02)
  expect_equal(extreme$conf.high, 1.0000, tolerance = 0.02)

  # 4/8 - 4/8
  responses <- c(rep(1, 4), rep(0, 4),rep(1, 4), rep(0, 4))
  arm <- rep(c("treat", "control"), times = c(8, 8))
  equal_prop <- ci_prop_diff_mn(x = responses, by = arm)

  expect_equal(equal_prop$conf.low, -0.4515, tolerance = 0.02)
  expect_equal(equal_prop$conf.high, 0.4515, tolerance = 0.02)

  # testing with data input
  df <- dplyr::tibble(response = resp,
               treat = trt
               )
  extreme_w_df <- ci_prop_diff_mn(response, treat, data = df)
  expect_equal(extreme_w_df, extreme)

  # Add print test
  expect_snapshot(
    ci_prop_diff_mn(response, treat, data = df)
  )
})

test_that("delta argument works", {
  resp <- expand(c(56, 48), c(70, 80))
  trt <- rep(c("a", "b"), times = c(70,80))

  single_del <- ci_prop_diff_mn(x = resp, by = trt, delta = -0.1)
  multi_del <- ci_prop_diff_mn(x = resp, by = trt, delta = c(-0.1, 0, 0.1))

  expect_equal(length(single_del$p.value), 1)
  expect_equal(length(multi_del$p.value), 3)
  expect_equal(single_del$p.value, multi_del$p.value[1])
})


test_that("ci_prop_diff_mn validates inputs correctly", {
  # Non-binary x
  expect_error(ci_prop_diff_mn(x = c(1, 2, 3), by = c("A", "B", "A")))

  # By with more than two levels
  expect_error(ci_prop_diff_mn(x = c(1, 0, 1), by = c("A", "B", "C")))

  # Mismatched lengths
  expect_error(ci_prop_diff_mn(x = c(1, 0, 1, 0), by = c("A", "B", "A")))

  # Invalid confidence level
  expect_error(ci_prop_diff_mn(x = c(1, 0), by = c("A", "B"), conf.level = 2))
  expect_error(ci_prop_diff_mn(x = c(1, 0), by = c("A", "B"), conf.level = 0))

  # Invalid delta
  expect_error(ci_prop_diff_mn(x = c(1, 0), by = c("A", "B"), delta = 2))
})

test_that("ci_prop_diff_mn_strata", {
  set.seed(123)
  trt<-c(rep(1, 100),rep(2, 100))
  response<-rbinom(200,1,.6)
  region <-1+rbinom(200,1,.5)
  sex <-1+rbinom(200,1,.5)

  exData<-data.frame(trt,response,region,sex)

  # SCORE METHOD
  strata <- interaction(region,sex)
  strata_mn <- ci_prop_diff_mn_strata(x = response, by = trt,
                         strata = strata, method = "score",
                         conf.level = 0.95, delta = -0.1)
  # The values are from running this same analysis in SAS
  expect_equal(strata_mn$conf.low, -0.16116, tolerance = 0.001)
  expect_equal(strata_mn$conf.high, 0.10954, tolerance = 0.001)

  # Check the values based on the delta = -0.1
  expect_equal(strata_mn$statistic, 1.05697, tolerance = 0.01)
  expect_equal(strata_mn$p.value, 0.14526, tolerance = 0.01)

  # SUMMARY SCORE METHOD
  # proc freq data=exRdata;
  # table region * sex * trt * response/COMMONRISKDIFF(CL=(score) column=2 test=(score));
  # ods output CommonPdiff=CommonPdiff CommonPdiffTests=CommonPdiffTests;
  # run;
  #
  # data output_mn;
  # merge CommonPdiff(where=(Method="Summary Score")) CommonPdiffTests(where=(Method="Summary Score"));
  # riskdiff=RiskDifference;
  # Z_inf=(RiskDifference--0.1)/StdErr; /*NI margin of -10%*/
  #   p_inf_1tail=1-cdf("NORMAL", Z_inf);
  # LowerCI=LowerCL;
  # UpperCI=UpperCL;
  # keep riskdiff Z_inf p_inf_1tail LowerCI UpperCI;
  # run;
  #
  # proc print data=output_mn; run;

  ss_strata_mn <- ci_prop_diff_mn_strata(x = response, by = trt,
                                      strata = strata, method = "summary score",
                                      conf.level = 0.95, delta = -0.1)

  # The values are from running this same analysis in SAS
  expect_equal(ss_strata_mn$conf.low, -0.15440, tolerance = 0.001)
  expect_equal(ss_strata_mn$conf.high, 0.10902, tolerance = 0.001)


  null_delta <- ci_prop_diff_mn_strata(x = response, by = trt,
                                      strata = strata, method = "score",
                                      conf.level = 0.95)
  expect_null(null_delta$statistic)
  expect_null(null_delta$p.value)

  vector_strata <- ci_prop_diff_mn_strata(x = response, by = trt,
                                      strata = c(region, sex),
                                      method = "score",
                                      conf.level = 0.95, delta = -0.1, data = exData)

  expect_equal(strata_mn, vector_strata)

  # Test improper strata length
  expect_error(
    ci_prop_diff_mn_strata(x = response, by = trt,
                           strata = c(region[1:54], sex),
                           method = "score",
                           conf.level = 0.95, delta = -0.1, data = exData)

  )


  # Test when strata and by are the same
  responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
  arm <- rep(c("treat", "control"), each = 20)
  strata <- rep(c("stratum1", "stratum2"), each = 20)

  # Calculate stratified confidence interval for difference in proportions
  expect_error(
    ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata)
  )
})

test_that("Test 0 response", {
  responses <- expand(c(3, 0), c(83, 46))
  arm <- rep(c("treat", "control"), c(83, 46))
  non_1_zero <- ci_prop_diff_mn(x = responses, by = arm)
  # Expected valeus from SAS
  expect_equal(non_1_zero$conf.low, -0.043, tolerance = 0.02)
  expect_equal(non_1_zero$conf.high, 0.101, tolerance = 0.02)


  # responses <- expand(c(9, 3, 7, 0), c(10, 83, 10, 46))
  # arm <- rep(c("treat", "control"), c(93, 56))
  # strata <- c(rep(c("stratum1", "stratum2"), c(10, 83)),
  #             rep(c("stratum1", "stratum2"), c(10, 46))
  # )
  # ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata)
  })

test_that("Data argument works", {
  set.seed(123)

  test_df <- data.frame(
    trt = c(rep(1, 100),rep(2, 100)),
    response = rbinom(200,1,.6),
    region = 1+rbinom(200,1,.5),
    sex = 1+rbinom(200,1,.5)
  )
  by_data <- ci_prop_diff_mn_strata(x = response,  by = trt, strata = c(region, sex),
                         method = "summary score", conf.level = 0.95, data=test_df)

  by_vector <- ci_prop_diff_mn_strata(x = test_df$response,  by = test_df$trt, strata = c(test_df$region, test_df$sex),
                                method = "summary score", conf.level = 0.95)


  expect_equal(by_data, by_vector)



})

test_that("More than 2 levels in strata",{
  set.seed(123)
  trt<-c(rep(1, 100),rep(2, 100))
  response<-rbinom(200,1,.6)
  region <-1+rbinom(200,1,.5)
  sex <-1+rbinom(200,1,.5)
  gender<-1+rbinom(200,1,.5)

  exData<-data.frame(trt,response,region,sex,gender)

  output<-ci_prop_diff_mn_strata(
    x=response,
    by=trt,
    strata=c(region,sex,gender),
    method = c("summary score"),
    conf.level = 0.95,
    delta = -0.1,
    data = exData
  )

  expect_equal(output$p.value, 0.1467, tolerance = 0.001)
  expect_equal(output$statistic, 1.051, tolerance = 0.001)
})

test_that("Test when strata group has only one event",{

set.seed(123)
trt<-c(rep(1, 100),rep(2, 100))
response<-rbinom(200,1,.6)
region <-1+rbinom(200,2,.8)
sex <-1+rbinom(200,1,.7)
gender<-1+rbinom(200,1,.7)

exData2<-data.frame(trt,response,region,sex,gender)

output<-ci_prop_diff_mn_strata(
  x=response,
  by=trt,
  strata=c(region,sex,gender),
  method = c("summary score"),
  conf.level = 0.95,
  delta = -0.1,
  data = exData2
)

output
expect_equal(output$conf.low, -0.1584, tolerance = 0.001)
})

test_that("Test when strata group is missing events", {

  set.seed(123)
  trt<-c(rep(1, 100),rep(2, 100))
  response<-rbinom(200,1,.6)
  region <-1+rbinom(200,1,.9)
  sex <-1+rbinom(200,1,.7)
  gender<-1+rbinom(200,1,.7)

  exData2<-data.frame(trt,response,region,sex,gender)

  output <- ci_prop_diff_mn_strata(
    x=response,
    by=trt,
    strata=c(region,sex,gender),
    method = c("summary score"),
    conf.level = 0.95,
    delta = -0.1,
    data = exData2
  )
  expect_equal(output$p.value, 0.1098, tolerance = 0.001)
  expect_equal(output$statistic, 1.228, tolerance = 0.001)

  output2 <- ci_prop_diff_mn_strata(
    x=response,
    by=trt,
    strata=c(region,sex,gender),
    method = c("score"),
    conf.level = 0.95,
    delta = -0.1,
    data = exData2
  )

  expect_equal(output$p.value, 0.1098, tolerance = 0.001)
  expect_equal(output$statistic, 1.228, tolerance = 0.001)

})

# Check print

    Code
      ci_prop_diff_mh_strata(x = results, by = treatment, strata = centre, data = agresti_long)
    Message
      
      -- Mantel-Haenszel Risk Difference Confidence Interval, Sato Variance ----------
      * 55/130 - 47/143
      * Estimate: 0.13
      * Variance: 0.003
      * 95% Confidence Interval:
        (0.0313, 0.2284)

---

    Code
      ci_prop_diff_mh_strata(x = results, by = treatment, strata = centre, sato_var = FALSE,
        data = agresti_long)
    Message
      
      -- Mantel-Haenszel Risk Difference Confidence Interval, Independent Binomial ---
      * 55/130 - 47/143
      * Estimate: 0.13
      * Variance: 0.002
      * 95% Confidence Interval:
        (0.0348, 0.2249)

---

    Code
      ci_rel_risk_cmh_strata(x = results, by = treatment, strata = centre, data = agresti_long)
    Message
      
      -- Mantel-Haenszel Common Relattive Risk Confidence Interval -------------------
      * 55/130 - 47/143
      * Estimate: 1.424
      * Variance: 0.02
      * 95% Confidence Interval:
        (1.0786, 1.8812)

# check the ci_prop_diff_nc_strata() function works

    Code
      ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "wilson",
        correct = FALSE)
    Message
      
      -- Stratified Newcombe Confidence Interval without continuity correction, Wilson
      * 50 responses out of 80
      * Weights: a.x = 0.211, b.x = 0.189, a.y = 0.118, b.y = 0.154, a.z = 0.174, b.z
      = 0.153
      * Estimate: 0.259
      * 95% Confidence Interval:
        (0.0392, 0.4502)

---

    Code
      ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "wilson",
        correct = TRUE)
    Message
      
      -- Stratified Newcombe Confidence Interval with continuity correction, Wilson --
      * 50 responses out of 80
      * Weights: a.x = 0.211, b.x = 0.189, a.y = 0.118, b.y = 0.154, a.z = 0.174, b.z
      = 0.153
      * Estimate: 0.259
      * 95% Confidence Interval:
        (0.0398, 0.4366)

---

    Code
      ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "cmh",
        correct = FALSE)
    Message
      
      -- Stratified Newcombe Confidence Interval without continuity correction, CMH --
      * 50 responses out of 80
      * Weights: a.x = 0.195, b.x = 0.163, a.y = 0.135, b.y = 0.138, a.z = 0.202, b.z
      = 0.167
      * Estimate: 0.254
      * 95% Confidence Interval:
        (0.0347, 0.4454)

---

    Code
      ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "cmh",
        correct = TRUE)
    Message
      
      -- Stratified Newcombe Confidence Interval with continuity correction, CMH -----
      * 50 responses out of 80
      * Weights: a.x = 0.195, b.x = 0.163, a.y = 0.135, b.y = 0.138, a.z = 0.202, b.z
      = 0.167
      * Estimate: 0.254
      * 95% Confidence Interval:
        (0.0362, 0.4314)

---

    Code
      ci_prop_diff_nc_strata(x = as.numeric(rsp), by = grp, strata = strata,
      weights_method = "wilson")
    Message
      
      -- Stratified Newcombe Confidence Interval without continuity correction, Wilson
      * 50 responses out of 80
      * Weights: a.x = 0.211, b.x = 0.189, a.y = 0.118, b.y = 0.154, a.z = 0.174, b.z
      = 0.153
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0)

---

    Code
      ci_prop_diff_nc_strata(x = as.numeric(rsp), by = grp, strata = strata,
      weights_method = "cmh")
    Message
      
      -- Stratified Newcombe Confidence Interval without continuity correction, CMH --
      * 50 responses out of 80
      * Weights: a.x = 0.195, b.x = 0.163, a.y = 0.135, b.y = 0.138, a.z = 0.202, b.z
      = 0.167
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0)

---

    Code
      ci_prop_diff_nc_strata(x = as.numeric(rsp), by = grp, strata = strata)
    Message
      
      -- Stratified Newcombe Confidence Interval without continuity correction, Wilson
      * 50 responses out of 80
      * Weights: a.x = 0.211, b.x = 0.189, a.y = 0.118, b.y = 0.154, a.z = 0.174, b.z
      = 0.153
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0)

---

    Code
      ci_prop_diff_nc_strata(x = rep_len(TRUE, length(rsp)), by = grp, strata = strata,
      weights_method = "wilson")
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! All values in `x` argument are either `TRUE` or `FALSE` and CI is not estimable.

---

    Code
      ci_prop_diff_nc_strata(x = rep_len(FALSE, length(rsp)), by = grp, strata = strata,
      weights_method = "cmh")
    Condition
      Error:
      ! All values in `x` argument are either `TRUE` or `FALSE` and CI is not estimable.

---

    Code
      ci_prop_diff_nc_strata(x = rsp, by = grp, strata = strata, weights_method = "wilson",
        correct = FALSE)
    Condition
      Warning:
      Less than 5 observations in some strata.
      Error in `ci_prop_diff_nc_strata()`:
      ! Arguments `unique(strata)` and `weights` must be the same length.

# ci_prop_diff_mn matches the values in the paper

    Code
      ci_prop_diff_mn(response, treat, data = df)
    Message
      
      -- Miettinen-Nurminen Confidence Interval --------------------------------------
      * 10/10 - 0/20
      * Estimate: 1
      * 95% Confidence Interval:
        (0.7156, 1)

# check the ci_prop_*() functions work

    Code
      wilson_dbl <- ci_prop_wilson(x_dbl, conf.level = 0.9, correct = FALSE)

---

    Code
      wilsoncc_dbl <- ci_prop_wilson(x_dbl, conf.level = 0.9, correct = TRUE)

---

    Code
      wilson_lgl <- ci_prop_wilson(x_lgl, conf.level = 0.9, correct = FALSE)

---

    Code
      ci_prop_wilson(x_rsp, conf.level = 0.9)
    Message
      
      -- Wilson Confidence Interval without continuity correction --------------------
      * 5 responses out of 10
      * Estimate: 0.5
      * 90% Confidence Interval:
        (0.2693, 0.7307)

---

    Code
      ci_prop_wilson(x_true)
    Message
      
      -- Wilson Confidence Interval without continuity correction --------------------
      * 32 responses out of 32
      * Estimate: 1
      * 95% Confidence Interval:
        (0.8928, 1)

---

    Code
      ci_prop_wilson(x_false)
    Message
      
      -- Wilson Confidence Interval without continuity correction --------------------
      * 0 responses out of 32
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0.1072)

---

    Code
      wald_dbl <- ci_prop_wald(x_dbl, conf.level = 0.9, correct = FALSE)

---

    Code
      waldcc_dbl <- ci_prop_wald(x_dbl, conf.level = 0.9, correct = TRUE)

---

    Code
      wald_lgl <- ci_prop_wald(x_lgl, conf.level = 0.9, correct = FALSE)

---

    Code
      ci_prop_wald(x_rsp, conf.level = 0.95, correct = TRUE)
    Message
      
      -- Wald Confidence Interval with Continuity Correction -------------------------
      * 5 responses out of 10
      * Estimate: 0.5
      * 95% Confidence Interval:
        (0.1401, 0.8599)

---

    Code
      ci_prop_wald(x_true)
    Message
      
      -- Wald Confidence Interval without Continuity Correction ----------------------
      * 32 responses out of 32
      * Estimate: 1
      * 95% Confidence Interval:
        (1, 1)

---

    Code
      ci_prop_wald(x_false)
    Message
      
      -- Wald Confidence Interval without Continuity Correction ----------------------
      * 0 responses out of 32
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0)

---

    Code
      clopper_pearson_dbl <- ci_prop_clopper_pearson(x_dbl, conf.level = 0.9)

---

    Code
      clopper_pearson_lgl <- ci_prop_clopper_pearson(x_lgl, conf.level = 0.9)

---

    Code
      ci_prop_clopper_pearson(x_rsp, conf.level = 0.95)
    Message
      
      -- Clopper-Pearson Confidence Interval -----------------------------------------
      * 5 responses out of 10
      * Estimate: 0.5
      * 95% Confidence Interval:
        (0.1871, 0.8129)

---

    Code
      ci_prop_clopper_pearson(x_true)
    Message
      
      -- Clopper-Pearson Confidence Interval -----------------------------------------
      * 32 responses out of 32
      * Estimate: 1
      * 95% Confidence Interval:
        (0.8911, 1)

---

    Code
      ci_prop_clopper_pearson(x_false)
    Message
      
      -- Clopper-Pearson Confidence Interval -----------------------------------------
      * 0 responses out of 32
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0.1089)

---

    Code
      agresti_coull_dbl <- ci_prop_agresti_coull(x_dbl, conf.level = 0.9)

---

    Code
      agresti_coull_lgl <- ci_prop_agresti_coull(x_lgl, conf.level = 0.9)

---

    Code
      ci_prop_agresti_coull(x_rsp, conf.level = 0.95)
    Message
      
      -- Agresti-Coull Confidence Interval -------------------------------------------
      * 5 responses out of 10
      * Estimate: 0.5
      * 95% Confidence Interval:
        (0.2366, 0.7634)

---

    Code
      ci_prop_agresti_coull(x_true)
    Message
      
      -- Agresti-Coull Confidence Interval -------------------------------------------
      * 32 responses out of 32
      * Estimate: 1
      * 95% Confidence Interval:
        (0.8727, 1)

---

    Code
      ci_prop_agresti_coull(x_false)
    Message
      
      -- Agresti-Coull Confidence Interval -------------------------------------------
      * 0 responses out of 32
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0.1273)

---

    Code
      jeffreys_dbl <- ci_prop_jeffreys(x_dbl, conf.level = 0.9)

---

    Code
      jeffreys_lgl <- ci_prop_jeffreys(x_lgl, conf.level = 0.9)

---

    Code
      ci_prop_jeffreys(x_rsp, conf.level = 0.95)
    Message
      
      -- Jeffreys Interval -----------------------------------------------------------
      * 5 responses out of 10
      * Estimate: 0.5
      * 95% Confidence Interval:
        (0.2235, 0.7765)

---

    Code
      ci_prop_jeffreys(x_true)
    Message
      
      -- Jeffreys Interval -----------------------------------------------------------
      * 32 responses out of 32
      * Estimate: 1
      * 95% Confidence Interval:
        (0.9251, 1)

---

    Code
      ci_prop_jeffreys(x_false)
    Message
      
      -- Jeffreys Interval -----------------------------------------------------------
      * 0 responses out of 32
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0.0749)

---

    Code
      mid_p_dbl <- ci_prop_mid_p(x_dbl, conf.level = 0.9)

---

    Code
      mid_p_lgl <- ci_prop_mid_p(x_lgl, conf.level = 0.9)

---

    Code
      ci_prop_mid_p(x_rsp, conf.level = 0.95)
    Message
      
      -- Mid-p Confidence Interval ---------------------------------------------------
      * 5 responses out of 10
      * Estimate: 0.5
      * 95% Confidence Interval:
        (0.212, 0.788)

---

    Code
      ci_prop_mid_p(x_true)
    Message
      
      -- Mid-p Confidence Interval ---------------------------------------------------
      * 32 responses out of 32
      * Estimate: 1
      * 95% Confidence Interval:
        (0.9106, 1)

---

    Code
      ci_prop_mid_p(x_false)
    Message
      
      -- Mid-p Confidence Interval ---------------------------------------------------
      * 0 responses out of 32
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0.0894)

---

    Code
      ci_prop_mid_p(rep(FALSE, 100))
    Message
      
      -- Mid-p Confidence Interval ---------------------------------------------------
      * 0 responses out of 100
      * Estimate: 0
      * 95% Confidence Interval:
        (0, 0.0295)

---

    Code
      ci_prop_mid_p(rep(TRUE, 100))
    Message
      
      -- Mid-p Confidence Interval ---------------------------------------------------
      * 100 responses out of 100
      * Estimate: 1
      * 95% Confidence Interval:
        (0.9705, 1)

---

    Code
      ci_prop_wilson(x_dbl, conf.level = c(0.9, 0.9))
    Condition
      Error in `ci_prop_wilson()`:
      ! The `conf.level` argument must be length 1.

---

    Code
      ci_prop_wilson(mtcars$cyl)
    Condition
      Error in `ci_prop_wilson()`:
      ! Expecting `x` to be either <logical> or <numeric/integer> coded as 0 and 1.

# check the ci_prop_strat_wilson() function works

    Code
      ci_prop_wilson_strata(x = rsp, strata = strata, weights = weights, correct = FALSE)
    Message
      
      -- Stratified Wilson Confidence Interval without continuity correction ---------
      * 50 responses out of 80
      * Weights: 0.048, 0.095, 0.143, 0.19, 0.238, 0.286
      * Estimate: 0.625
      * 95% Confidence Interval:
        (0.4867, 0.7186)

---

    Code
      ci_prop_wilson_strata(x = rsp, strata = strata, weights = weights, correct = TRUE)
    Message
      
      -- Stratified Wilson Confidence Interval with continuity correction ------------
      * 50 responses out of 80
      * Weights: 0.048, 0.095, 0.143, 0.19, 0.238, 0.286
      * Estimate: 0.625
      * 95% Confidence Interval:
        (0.4483, 0.7531)

---

    Code
      ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, weights = weights)
    Message
      
      -- Stratified Wilson Confidence Interval without continuity correction ---------
      * 50 responses out of 80
      * Weights: 0.048, 0.095, 0.143, 0.19, 0.238, 0.286
      * Estimate: 0.625
      * 95% Confidence Interval:
        (0.4867, 0.7186)

---

    Code
      ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata)
    Message
      
      -- Stratified Wilson Confidence Interval without continuity correction ---------
      * 50 responses out of 80
      * Weights: a.x = 0.211, b.x = 0.189, a.y = 0.118, b.y = 0.154, a.z = 0.174, b.z
      = 0.153
      * Estimate: 0.625
      * 95% Confidence Interval:
        (0.5242, 0.7269)

---

    Code
      ci_prop_wilson_strata(x = rep_len(TRUE, length(rsp)), strata = strata, weights = weights)
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! All values in `x` argument are either `TRUE` or `FALSE` and CI is not estimable.

---

    Code
      ci_prop_wilson_strata(x = rep_len(FALSE, length(rsp)), strata = strata,
      weights = weights)
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! All values in `x` argument are either `TRUE` or `FALSE` and CI is not estimable.

---

    Code
      ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, max.iterations = -1)
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! Argument `max.iterations` must be a positive integer.

---

    Code
      ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, max.iterations = -1)
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! Argument `max.iterations` must be a positive integer.

---

    Code
      ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, weights = weights +
        pi / 5)
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! The sum of the `weights` argument must be 1

---

    Code
      ci_prop_wilson_strata(x = as.numeric(rsp), strata = strata, weights = weights +
        pi)
    Condition
      Error in `ci_prop_wilson_strata()`:
      ! The `weights` argument must be in the interval `[0, 1]`.

# Test Print

    Code
      ci_prop_diff_wald(response, treat)
    Message
      
      -- Wald Confidence Interval without Continuity Correction ----------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0575, 0.3425)

---

    Code
      ci_prop_diff_wald(response, treat, correct = TRUE)
    Message
      
      -- Wald Confidence Interval with Continuity Correction -------------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0441, 0.3559)

---

    Code
      ci_prop_diff_haldane(response, treat)
    Message
      
      -- Haldane Confidence Interval -------------------------------------------------
      * 56/70 - 48/80
      * Estimate: 0.194
      * 95% Confidence Interval:
        (0.0535, 0.3351)

---

    Code
      ci_prop_diff_jp(response, treat)
    Message
      
      -- Jeffreys-Perks Confidence Interval ------------------------------------------
      * 56/70 - 48/80
      * Estimate: 0.194
      * 95% Confidence Interval:
        (0.0531, 0.3355)

---

    Code
      ci_prop_diff_mee(response, treat)
    Message
      
      -- Mee Confidence Interval -----------------------------------------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0533, 0.3377)

---

    Code
      print(ci_prop_diff_mee(response, treat, delta = c(0, 0.1)))
    Message
      
      -- Mee Confidence Interval -----------------------------------------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0533, 0.3377)
      
      -- Delta 
      * At 0 the statistic is 2.65 and the p-value is 0.004
      * At 0.1 the statistic is 1.348 and the p-value is 0.0889

---

    Code
      ci_prop_diff_ha(response, treat)
    Message
      
      -- Anderson-Hauck Confidence Interval ------------------------------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0494, 0.3506)

---

    Code
      ci_prop_diff_nc(response, treat, correct = FALSE)
    Message
      
      -- Newcombe Confidence Interval without continuity correction ------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0524, 0.3339)

---

    Code
      print(ci_prop_diff_nc(response, treat, correct = TRUE))
    Message
      
      -- Newcombe Confidence Interval with continuity correction ---------------------
      * 56/70 - 48/80
      * Estimate: 0.2
      * 95% Confidence Interval:
        (0.0428, 0.3422)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_common_risk.R
\name{ci_rel_risk_cmh_strata}
\alias{ci_rel_risk_cmh_strata}
\title{Mantel-Haenszel Stratified Relative Risk Confidence Interval}
\usage{
ci_rel_risk_cmh_strata(x, by, strata, conf.level = 0.95, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{strata}{(\code{numeric}) \cr A vector specifying the stratum for each observation. It needs
to be the length of x or a multiple of x if multiple levels of strata are
present. Can also be a column name (or vector of column names NOT quoted)
if a data frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{estimate}{The Mantel-Haenszel estimated common risk difference}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{variance}{Mantel-Haenszel variance estimate \eqn{Var(log(RR_MH))}}
\item{method}{Description of the method used ("Mantel-Haenszel Common Relative Risk Confidence Interval")}
}
\description{
Calculates the confidence interval for the Mantel-Haenszel estimate of the
common relative risk across multiple 2x2 tables (strata)
}
\details{
The Mantel-Haenszel relative risk difference is computed as:

\deqn{RR_{MH} = \frac{\sum_{k} s_{xk}~n_{yk}/N_k}{\sum_{k} s_{yk}~n_{xk}/N_k}}

The variance is:

\deqn{\hat{\sigma}^2 = \hat{Var}(log(RR_{MH})) =
\frac{\sum_{k}(n_{xk}~n_{yk}~(s_{xk}+s_{yk}) - s_{xk}~s_{yk}~N_k)/N_k^2}
{(\sum_{k}s_{xk}~n_{yk}/N_k)(\sum_{k}s_{yk}~n_{xk}/N_k)}}

The confidence interval is then \eqn{\left(RR_{MH}\times exp(-z_{1-\alpha/2} \sqrt{\hat{\sigma}^2},
RR_{MH}\times exp(z_{1-\alpha/2} \sqrt{\hat{\sigma}^2}\right)}.
}
\examples{
# Generate binary samples with strata
responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
arm <- rep(c("treat", "control"), 20)
strata <- rep(c("stratum1", "stratum2"), times = c(20, 20))

# Calculate common risk difference
ci_rel_risk_cmh_strata(x = responses, by = arm, strata = strata)
}
\references{
Agresti, A. (2013). Categorical Data Analysis. 3rd Edition. John Wiley & Sons, Hoboken, NJ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_diff.R
\name{ci_prop_diff_wald}
\alias{ci_prop_diff_wald}
\title{Wald Confidence Interval for Difference in Proportions}
\usage{
ci_prop_diff_wald(x, by, conf.level = 0.95, correct = FALSE, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{correct}{(\code{logical})\cr apply continuity correction.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:
\item{n}{Number of responses in each by group}
\item{N}{Total number in each by group}
\item{estimate}{The point estimate of the difference in proportions (p_1 - p_2)}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Type of method used}
}
\description{
Calculates the Wald interval by following the usual textbook definition
for a difference in proportions confidence interval using the normal approximation.
}
\details{
\deqn{(\hat{p}_1 - \hat{p}_2) \pm z_{\alpha/2}
\sqrt{\frac{\hat{p}_1(1 - \hat{p}_1)}{n_1}+\frac{\hat{p}_2(1 - \hat{p}_2)}{n_2}}}
}
\examples{
responses <- expand(c(9, 3), c(10, 10))
arm <- rep(c("treat", "control"), times = c(10, 10))

# Calculate 95\% confidence interval for difference in proportions
ci_prop_diff_wald(x = responses, by = arm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{expand}
\alias{expand}
\title{Expand Count Data into Binary Vectors}
\usage{
expand(x, n)
}
\arguments{
\item{x}{Integer (or vector of integers) representing the number of successes.}

\item{n}{Integer (or vector of integers) representing the total number of participants.}
}
\value{
A logical vector where TRUE represents a success and FALSE represents a failure.
The length of the vector equals the sum of all sample sizes.
}
\description{
Converts count data (number of successes and total sample size) into a binary vector
of TRUE/FALSE values. This is useful for converting summary statistics back into
raw data format for analysis functions that require individual-level data.
}
\details{
For each pair of values in \code{x} and \code{n}, the function creates a vector with \code{x} TRUE values
followed by \code{n-x} FALSE values. If multiple pairs are provided, the resulting vectors are
concatenated in order.
}
\examples{
# Convert 4 successes out of 13 participants to binary data
expand(4, 13)

# Convert multiple groups of data
# Group 1: 9 successes out of 10
# Group 2: 3 successes out of 10
expand(c(9, 3), c(10, 10))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci.R
\name{ci_prop_agresti_coull}
\alias{ci_prop_agresti_coull}
\title{Agresti-Coull CI}
\usage{
ci_prop_agresti_coull(x, conf.level = 0.95, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{Number of responses}
\item{N}{Total number}
\item{estimate}{The point estimate of the proportion}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Type of method used}
}
\description{
Calculates the \code{Agresti-Coull} interval (created by \verb{Alan Agresti} and \verb{Brent Coull}) by
(for 95\% CI) adding two successes and two failures to the data and then using the Wald formula to construct a CI.
}
\details{
\deqn{
\left( \frac{\tilde{p} + z^2_{\alpha/2}/2}{n + z^2_{\alpha/2}} \pm
z_{\alpha/2} \sqrt{\frac{\tilde{p}(1 - \tilde{p})}{n} +
\frac{z^2_{\alpha/2}}{4n^2}} \right)}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_common_risk.R
\name{ci_prop_diff_mh_strata}
\alias{ci_prop_diff_mh_strata}
\title{Mantel-Haenszel Common Risk Difference Confidence Interval}
\usage{
ci_prop_diff_mh_strata(
  x,
  by,
  strata,
  conf.level = 0.95,
  sato_var = TRUE,
  data = NULL
)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{strata}{(\code{numeric}) \cr A vector specifying the stratum for each observation. It needs
to be the length of x or a multiple of x if multiple levels of strata are
present. Can also be a column name (or vector of column names NOT quoted)
if a data frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{sato_var}{(\code{logical})\cr Use Sato variance estimate}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{estimate}{The Mantel-Haenszel estimated common risk difference}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{variance}{Variance estimate}
\item{statistic}{Z-Statistic under the null hypothesis, assuming a common risk difference of 0}
\item{p.value}{p-value under the null hypothesis, assuming a common risk difference of 0}
\item{method}{Description of the method used ("Mantel-Haenszel Confidence Interval, Sato Variance")
or ("Mantel-Haenszel Confidence Interval, Independent Binomial") }
}
\description{
Calculates the confidence interval for the Mantel-Haenszel estimate of the
common risk difference across multiple 2x2 tables (strata), using the Sato
or Independent Binomial variance estimator.
}
\details{
The Mantel-Haenszel common risk difference is computed as:

\deqn{\hat{\delta}_{MH} = \frac{\sum_{k} w_k \hat{\delta}_k }{\sum_{k} w_k}}

where \eqn{w_k = \frac{n_{xk} n_{yk}}{N_k}},
\eqn{\hat{\delta}_k = s_{xk}/n_{xk} - y_{yk}/n_{yk}},
\eqn{N_k = n_{xk} + n_{yk}},
\eqn{s_{xk}} and \eqn{s_{yk}} are the number of events in each group, and \eqn{n_{xk}},
and \eqn{n_{yk}} are the group sizes in stratum \eqn{k}.

The Sato variance is:

\deqn{\hat{\sigma}^2(\hat{\delta}_{MH}) = \frac{\hat{d}_{MH} \sum_{k}{P_k} + \sum_k Q_k}{\left( \sum_k w_k \right)^2}}

where \eqn{P_k = \frac{n_{xk}^2 s_{yk} - n_{yk}^2 s_{xk} + n_{xk} n_{yk} (n_{yk} - n_{xk})/2}{N_k^2}}
and \eqn{Q_k = \frac{s_{xk}(n_{yk} - s_{yk}) + s_{yk}(n_{xk} - s_{xk})}{2 N_k}}.

The Cochran Independent Binomial variance is:

\deqn{\hat{\sigma}^2(\hat{\delta}_{C}) = \sum_{k} w_k^2 \left[
\frac{\hat{p}_{1k}(1 - \hat{p}_{1k})}{n_{1k}} +
\frac{\hat{p}_{2k}(1 - \hat{p}_{2k})}{n_{2k}}
\right]}

where \eqn{\hat{p}_{1k} = \frac{s_{xk}}{n_{xk}}}
and \eqn{\hat{p}_{2k} = \frac{s_{yk}}{n_{yk}}}.

The confidence interval is then \eqn{\hat{\delta}_{MH} \pm z_{1-\alpha/2} \sqrt{\hat{\sigma}^2(\hat{d}_{MH})}}.
}
\examples{
# Generate binary samples with strata
responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
arm <- rep(c("treat", "control"), 20)
strata <- rep(c("stratum1", "stratum2"), times = c(20, 20))

# Calculate common risk difference
ci_prop_diff_mh_strata(x = responses, by = arm, strata = strata)
# Calculate risk difference with independent binomial variance
ci_prop_diff_mh_strata(x = responses, by = arm, strata = strata, sato_var = FALSE)
}
\references{
Agresti, A. (2013). Categorical Data Analysis. 3rd Edition. John Wiley & Sons, Hoboken, NJ p. 231
Cochran, W.G. (1954). The Combination of estimates from different experiments. Biometrics, 10(1), p.101-129
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_diff.R
\name{ci_prop_diff_nc}
\alias{ci_prop_diff_nc}
\title{Newcombe Confidence Interval for Difference in Proportions}
\usage{
ci_prop_diff_nc(x, by, conf.level = 0.95, correct = FALSE, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{correct}{(\code{logical})\cr apply continuity correction.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{The number of responses for each group}
\item{N}{The total number in each group}
\item{estimate}{The point estimate of the difference in proportions}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Anderson-Hauck Confidence Interval}
}
\description{
Newcombe Confidence Interval for Difference in Proportions
}
\details{
The \strong{Wilson (Score)} confidence limits without continuity correction for each
individual binomial proportion, \eqn{p_i = x_i / n_i}, for \eqn{i = 1, 2}, are given by:

\deqn{
\frac{
  (2 n_i \hat{p}_i + z^2)
  \pm
  z \sqrt{ 4 n_i \hat{p}_i (1 - \hat{p}_i) + z^2 }
}{
  2 (n_i + z^2)
}
}

Denote the lower and upper Wilson (Score) confidence limits for \eqn{p_i} as
\eqn{L_i} and \eqn{U_i}, respectively.

Then, the \strong{Newcombe (Score)} confidence limits for the difference in proportions
(\eqn{p_1 - p_2}) are given by:

\deqn{
\text{Lower limit: } (\hat{p}_1 - \hat{p}_2) -
\sqrt{ (\hat{p}_1 - L_1)^2 + (U_2 - \hat{p}_2)^2 }
}

\deqn{
\text{Upper limit: } (\hat{p}_1 - \hat{p}_2) +
\sqrt{ (U_1 - \hat{p}_1)^2 + (\hat{p}_2 - L_2)^2 }
}

The confidence intervals with continuity correction for each individual binomial proportion are
obtained using the \strong{Wilson (Score) confidence limits with continuity correction}.

For each binomial proportion \eqn{p_i = x_i / n_i}, where \eqn{i = 1, 2},
the confidence intervals are given by:

\deqn{
\frac{ 2 n_i \hat{p}_i + z^2 }{ 2 (n_i + z^2) }
\; \pm \;
\frac{ z }{ 2 (n_i + z^2) }
\sqrt{
  z^2 - \frac{2}{n_i} + 4 \hat{p}_i
  \left[ n_i (1 - \hat{p}_i) + 1 \right]
}
}
}
\examples{
responses <- expand(c(9, 3), c(10, 10))
arm <- rep(c("treat", "control"), times = c(10, 10))

# Calculate 95\% confidence interval for difference in proportions
ci_prop_diff_nc(x = responses, by = arm)
}
\references{
Newcombe, R. G. (1998). Interval estimation for the difference between
independent proportions: Comparison of eleven methods. \emph{Statistics in Medicine, 17}(8), 873–890.
\href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_mn.R
\name{ci_prop_diff_mn_strata}
\alias{ci_prop_diff_mn_strata}
\title{Stratified Miettinen-Nurminen Confidence Interval for Difference in Proportions}
\usage{
ci_prop_diff_mn_strata(
  x,
  by,
  strata,
  method = c("score", "summary score"),
  conf.level = 0.95,
  delta = NULL,
  data = NULL
)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{strata}{(\code{numeric}) \cr A vector specifying the stratum for each observation. It needs
to be the length of x or a multiple of x if multiple levels of strata are
present. Can also be a column name (or vector of column names NOT quoted)
if a data frame provided in the \code{data} argument.}

\item{method}{(\code{string}) \cr Specifying how the CIs should be calculated. It must
equal either 'score' or 'summary score'. See details for more information
about the implementation differences.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{delta}{(\code{numeric}) \cr Optionally a single number or a vector of
numbers between -1 and 1 (not inclusive) to set the difference between two
groups under the null hypothesis. If provided, the function returns the
test statistic and p-value under the \code{delta}
hypothesis.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{estimate}{The point estimate of the difference in proportions (p_x -
p_y)} \item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used} \item{delta}{delta value(s) used}
\item{statistic}{Z-Statistic under the null hypothesis based on the given 'delta'}
\item{p.value}{p-value under the null hypothesis based on the given 'delta'}
\item{method}{Description of the method used ("Stratified \{method\}
Miettinen-Nurminen Confidence Interval")}

If \code{delta} is not provided statistic and p.value will be NULL
}
\description{
Calculates Stratified Miettinen-Nurminen (MN) confidence intervals and
corresponding point estimates for the difference between two proportions
}
\details{
The function implements the stratified Miettinen-Nurminen method to compute
confidence intervals for the difference between two proportions across multiple strata.
\deqn{H_0: \hat{d}-\delta <= 0 \qquad \text{vs.} \qquad H_1: \hat{d}-\delta > 0}

The "score" method is a weighted MN score first described in the original 1985
paper. The formula is:
\itemize{
\item Calculates weights for each stratum as \eqn{w_i = \frac{n_{xi} \cdot n_{yi}}{n_{xi} + n_{yi}}}
\item Computes the overall weighted difference \eqn{\hat{d} = \frac{\sum w_i \hat{p}_{xi}}{\sum w_i} -
          \frac{\sum w_i \hat{p}_{yi}}{\sum w_i}}
\item Uses the stratified test statistic: \deqn{Z_{\delta} = \frac{\hat{d} - \delta}
          {\sqrt{\sum_{i=1}^k \left(\frac{w_i}{\sum w_i}\right)^2 \cdot \hat{\sigma}_{mn}^2({d})}}}
\item Finds the range of all values of \eqn{\delta} for which the stratified test statistic (\eqn{Z_\delta})
falls in the acceptance region \eqn{\{ Z_\delta < z_{\alpha/2}\}}
}

The \eqn{\hat{\sigma}_{mn}^2(\hat{d})} is the Miettinen-Nurminen variance estimate.
See the details of \code{\link[=ci_prop_diff_mn]{ci_prop_diff_mn()}} for how \eqn{\hat{\sigma}_{mn}^2(\delta)} is calculated.

The "summary score" method follows the meta-analyses proposed in Agresti
2013 and is consistent with the "Summary Score Confidence Limits" method
used in SAS. The formula is:
\itemize{
\item The point estimate of the stratified risk difference is a weighted average of the midpoints of the within-stratum MN confidence intervals:
\deqn{
      \hat{d}_{\text{S}} = \sum_i \hat{d}_i w_i
      }
\item Define \eqn{s_i} as the width of the CI for the \eqn{i}th stratum divided by \eqn{2 \times z_{\alpha/2}} and then stratum weights are given by
\deqn{
    w_i = \left( \frac{1}{s_i^2} \right) \bigg/ \sum_i \left( \frac{1}{s_i^2} \right)
    }
\item The variance of \eqn{\hat{d}_{\text{S}} } is computed as
\deqn{
     \widehat{\text{Var}}(\hat{d}_{\text{S}}) = \frac{1}{\sum_i \left( \frac{1}{s_i^2} \right) }
    }
\item Confidence limits for the stratified risk difference estimate are
\deqn{
    \hat{d}_{\text{S}} \pm \left( z_{\alpha /2} \times \widehat{\text{Var}}(\hat{d}_{\text{S}}) \right)
    }
}
}
\examples{
# Generate binary samples with strata
responses <- expand(c(9, 3, 7, 2), c(10, 10, 10, 10))
arm <- rep(c("treat", "control"), 20)
strata <- rep(c("stratum1", "stratum2"), times = c(20, 20))

# Calculate stratified confidence interval for difference in proportions
ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata)

# Using the summary score method
ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata,
                      method = "summary score")

# Calculate 99\% confidence interval
ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata,
                      conf.level = 0.99)

# Calculate p-value under null hypothesis delta = 0.2
ci_prop_diff_mn_strata(x = responses, by = arm, strata = strata,
                      delta = 0.2)

}
\references{
Miettinen, O. S., & Nurminen, M. (1985). Comparative analysis of two rates.
Statistics in Medicine, 4(2), 213-226.

\href{https://documentation.sas.com/doc/en/pgmsascdc/9.4_3.5/procstat/procstat_freq_details63.htm}{Common Risk Difference :: Base SAS(R) 9.4 Procedures Guide: Statistical Procedures, Third Edition}

Agresti, A. (2013). Categorical Data Analysis. 3rd Edition. John Wiley & Sons, Hoboken, NJ
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_diff.R
\name{ci_prop_diff_jp}
\alias{ci_prop_diff_jp}
\title{Jeffreys-Perks Confidence Interval for Difference in Proportions}
\usage{
ci_prop_diff_jp(x, by, conf.level = 0.95, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{The number of responses for each group}
\item{N}{The total number in each group}
\item{estimate}{The point estimate of the difference in proportions (theta*)}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Jeffreys-Perks Confidence Interval}
}
\description{
Jeffreys-Perks Confidence Interval for Difference in Proportions
}
\details{
The confidence interval is calculated by \eqn{\theta^* \pm w} where:

\deqn{\theta^* = \frac{(\hat{p}_1 - \hat{p}_2) + z^2v(1-2\hat{\psi})}{1+z^2u}}
where
\deqn{w = \frac{z}{1+z^2u}\sqrt{u\{4\hat{\psi}(1-\hat{\psi})-(\hat{p}_1 - \hat{p}_2)^2\}+2v(1-2\hat{\psi})(\hat{p}_1-\hat{p}_2)
+4z^2v^2(1-2\hat{\psi})^2
}}
\deqn{\hat{\psi} = \frac{1}{2}\left(\frac{x_1 + 1/2}{n_1+1}+\frac{x_2 + 1/2}{n_2+1}\right)}
\deqn{u = \frac{1/n_1 + 1/n_2}{4}}
\deqn{v = \frac{1/n_1 - 1/n_2}{4}}
}
\examples{
responses <- expand(c(9, 3), c(10, 10))
arm <- rep(c("treat", "control"), times = c(10, 10))

# Calculate 95\% confidence interval for difference in proportions
ci_prop_diff_jp(x = responses, by = arm)
}
\references{
\href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_mn.R
\name{ci_prop_diff_mn}
\alias{ci_prop_diff_mn}
\title{Miettinen-Nurminen Confidence Interval for Difference in Proportions}
\usage{
ci_prop_diff_mn(x, by, conf.level = 0.95, delta = NULL, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{delta}{(\code{numeric}) \cr Optionally a single number or a vector of
numbers between -1 and 1 (not inclusive) to set the difference between two
groups under the null hypothesis. If provided, the function returns the
test statistic and p-value under the \code{delta}
hypothesis.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{estimate}{The point estimate of the difference in proportions (p_x - p_y)}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{delta}{delta value(s) used}
\item{statistic}{Z-Statistic under the null hypothesis based on the given 'delta'}
\item{p.value}{p-value under the null hypothesis based on the given 'delta'}
\item{method}{Description of the method used ("Miettinen-Nurminen Confidence Interval")}

If \code{delta} is not provided statistic and p.value will be NULL
}
\description{
Calculates the Miettinen-Nurminen (MN) confidence interval for the difference
between two proportions. This method can be more accurate than traditional
methods, especially with small sample sizes or proportions close to 0 or 1.
}
\details{
The function implements the Miettinen-Nurminen method to compute
confidence intervals for the difference between two proportions. This approach:
\itemize{
\item Calculates the Miettinen-Nurminen score test statistic for different
possible values of the proportion difference (delta)
\item Identifies the delta values where the test statistic equals the critical
value corresponding to the desired confidence level
\item Returns these boundary values as the confidence interval limits
}

The method uses a score test with a small-sample correction factor, making it
more accurate than normal approximation methods, especially for small samples
or extreme proportions. The equation for the test statistics is as follows:

\deqn{H_0: \hat{d}-\delta <= 0 \qquad \text{vs.} \qquad H_1: \hat{d}-\delta > 0}

\deqn{ T_\delta = \frac{\hat{p_x} - \hat{p_y} - \delta}{\sigma_{mn}(\delta)}}

where \eqn{\hat{p_*} = s_*/n_*} represent the observed number of successes
divided by the number of participant in that group. The \eqn{\sigma_{mn}(\delta)} is a
function of the delta values and is create with the following equation"
\eqn{\tilde{p_*}} represent the MLE of the proportions.
\deqn{
\sigma_{mn}(\delta) = \sqrt{\left[\frac{\tilde{p_y}(1-\tilde{p_y})}{n_x}+\frac{\tilde{p_x}(1-\tilde{p_x})}{n_y} \right]\left(\frac{N}{N-1}\right)}
}
\eqn{ \tilde{p_x} = 2p\cdot{cos(a)} - \frac{L_2}{3L_3}} and \eqn{ \tilde{p_y} = \tilde{p_x} + \delta}
where:
\itemize{
\item \eqn{p = \pm \sqrt{\frac{L_2^2}{(3L_3)^2} - \frac{L_1}{3L_3}}}
\item \eqn{a = 1/3[\pi + cos^{-1}(q/p^3)]}
\item \eqn{q = \frac{L_2^3}{(3L_3)^3} - \frac{L_1L_2}{6L_3^2} + \frac{L_0}{2L_3}}
\item \eqn{L_3 = n_x + n_y }
\item \eqn{L_2 = (n_x + 2 n_y)\delta - N - (s_x + s_y)}
\item \eqn{L_1 = (n_y\delta - L_3 - 2s_y)\delta + s_x + s_y}
\item \eqn{L_0 = s_y\delta(1-\delta)}
}

For more information about these equations see Miettinen (1985)
}
\examples{
# Generate binary samples
responses <- expand(c(9, 3), c(10, 10))
arm <- rep(c("treat", "control"), times = c(10, 10))

# Calculate 95\% confidence interval for difference in proportions
ci_prop_diff_mn(x = responses, by = arm)

# Calculate 99\% confidence interval
ci_prop_diff_mn(x = responses, by = arm, conf.level = 0.99)

# Calculate the p-value under the null hypothesis delta = -0.1
ci_prop_diff_mn(x = responses, by = arm, delta = -0.1)

# Calculate from a data.frame
data <- data.frame(responses, arm)
ci_prop_diff_mn(x = responses, by = arm, data = data)
}
\references{
Miettinen, O. S., & Nurminen, M. (1985). Comparative analysis of
two rates. Statistics in Medicine, 4(2), 213-226.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{combine_strata}
\alias{combine_strata}
\title{Function to combine strata via interaction if strata is passed as a vector}
\usage{
combine_strata(x, strata)
}
\description{
Function to combine strata via interaction if strata is passed as a vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_common_risk.R
\name{ci_prop_diff_nc_strata}
\alias{ci_prop_diff_nc_strata}
\title{Stratified Newcombe Common Risk Difference Confidence Interval}
\usage{
ci_prop_diff_nc_strata(
  x,
  by,
  strata,
  conf.level = 0.95,
  correct = FALSE,
  weights_method = c("wilson", "cmh"),
  data = NULL
)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{strata}{(\code{numeric}) \cr A vector specifying the stratum for each observation. It needs
to be the length of x or a multiple of x if multiple levels of strata are
present. Can also be a column name (or vector of column names NOT quoted)
if a data frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{correct}{(scalar \code{logical})\cr include the continuity correction. For further information, see for example
[ci_prop_diff_nc())].

[ci_prop_diff_nc())]: R:ci_prop_diff_nc())}

\item{weights_method}{(\code{character})\cr Can be either "wilson" or "cmh" and directs the way weights are estimated.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{Number of responses}
\item{N}{Total number}
\item{estimate}{The point estimate of the proportion}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{weights}{Weights of each strata calculated as per the specified "weights_method" argument.}
\item{method}{Type of method used}
}
\description{
Calculates the stratified Newcombe confidence interval for unequal proportions
as described in Yan X, Su XG. Stratified Wilson and Newcombe confidence intervals
or multiple binomial proportions. Weights are estimated using CMH or Wilson methods.
}
\details{
\deqn{
L = \hat{d}_{\rm MH} - z_{\alpha/2} \sqrt{
  \sum_h w_h^2 L_{2h} (1 - L_{2h}) +
  \sum_h w_h^2 U_{1h} (1 - U_{1h})
}
}

\deqn{
U = \hat{d}_{\rm MH} + z_{\alpha/2} \sqrt{
  \sum_h w_h^2 L_{2h} (1 - L_{2h}) +
  \sum_h w_h^2 U_{1h} (1 - U_{1h})
}
}

Where:
\itemize{
\item \eqn{\hat{d}_{\rm MH}}: Mantel-Haenszel common risk difference
\item \eqn{z_{\alpha/2}}: standard normal critical value
\item \eqn{w_h}: stratum weights
\item \eqn{L_{2h}}, \eqn{U_{1h}}: Wilson-type CI limits for stratum h
}
}
\examples{
set.seed(1)
rsp <- sample(c(TRUE, FALSE), 100, TRUE)
grp <- sample(c("Placebo", "Treatment"), 100, TRUE)
strata_data <- data.frame(
  "f1" = sample(c("a", "b"), 100, TRUE),
  "f2" = sample(c("x", "y", "z"), 100, TRUE),
  stringsAsFactors = TRUE
)
strata <- interaction(strata_data)

ci_prop_diff_nc_strata(
  x = rsp, by = grp, strata = strata, weights_method = "cmh",
  conf.level = 0.95
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci.R
\name{ci_prop_wald}
\alias{ci_prop_wald}
\title{Wald CI}
\usage{
ci_prop_wald(x, conf.level = 0.95, correct = FALSE, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{correct}{(\code{logical})\cr apply continuity correction.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{Number of responses}
\item{N}{Total number}
\item{estimate}{The point estimate of the proportion}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Type of method used}
}
\description{
Calculates the Wald interval by following the usual textbook definition
for a single proportion confidence interval using the normal approximation.
}
\details{
\deqn{\hat{p} \pm z_{\alpha/2} \sqrt{\frac{\hat{p}(1 - \hat{p})}{n}}}
}
\examples{
# example code
x <- c(
TRUE, TRUE, TRUE, TRUE, TRUE,
FALSE, FALSE, FALSE, FALSE, FALSE
)

ci_prop_wald(x, conf.level = 0.9)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci.R
\name{ci_prop_clopper_pearson}
\alias{ci_prop_clopper_pearson}
\title{Clopper-Pearson CI}
\usage{
ci_prop_clopper_pearson(x, conf.level = 0.95, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{Number of responses}
\item{N}{Total number}
\item{estimate}{The point estimate of the proportion}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Type of method used}
}
\description{
Calculates the Clopper-Pearson interval by calling \code{\link[stats:binom.test]{stats::binom.test()}}.
Also referred to as the \code{exact} method.
}
\details{
\deqn{
\left(1 + \frac{N-n+1}{nF[\frac{\alpha}{2};2n,2(N-n+1)]} \right)^{-1}, \left( 1 + \frac{N-n}{(n+1)F[1-\frac{\alpha}{2};2(n+1),2(N-n)]} \right)^{-1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci.R
\name{ci_prop_mid_p}
\alias{ci_prop_mid_p}
\title{Mid-P CI}
\usage{
ci_prop_mid_p(x, conf.level = 0.95, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{Number of responses}
\item{N}{Total number}
\item{estimate}{The point estimate of the proportion}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Type of method used}
}
\description{
Calculates the exact mid-p CI for binomial proportions by inverting two one-sided binomial tests that include the mid-p tail.
This is calculated by finding the \eqn{P_L} and \eqn{P_U} that satisfies the following equations:
\deqn{\sum _{x=n_1+1}^{n} \binom {n}{x} P_{L}^{x}(1-P_{L})^{n-x} + \frac{1}{2} \binom{n}{n_1} P_{L}^{n_1}(1-P_{L})^{n-n_1} = \alpha /2}
\deqn{\sum _{x=0}^{n_1-1} \binom {n}{x} P_{U}^{x}(1-P_{U})^{n-x} + \frac{1}{2} \binom{n}{n_1} P_{U}^{n_1}(1-P_{U})^{n-n_1} = \alpha /2}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_counts}
\alias{get_counts}
\title{To get the n's and response totals with out without strata}
\usage{
get_counts(x, by, strata = 1)
}
\description{
To get the n's and response totals with out without strata
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci.R
\name{ci_prop_wilson_strata}
\alias{ci_prop_wilson_strata}
\title{Stratified Wilson CI}
\usage{
ci_prop_wilson_strata(
  x,
  strata,
  weights = NULL,
  conf.level = 0.95,
  max.iterations = 10L,
  correct = FALSE,
  data = NULL
)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{strata}{(\code{numeric}) \cr A vector specifying the stratum for each observation. It needs
to be the length of x or a multiple of x if multiple levels of strata are
present. Can also be a column name (or vector of column names NOT quoted)
if a data frame provided in the \code{data} argument.}

\item{weights}{(\code{numeric})\cr weights for each level of the strata. If \code{NULL}, they are
estimated using the iterative algorithm that
minimizes the weighted squared length of the confidence interval.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{max.iterations}{(positive \code{integer})\cr maximum number of iterations for the iterative procedure used
to find estimates of optimal weights.}

\item{correct}{(scalar \code{logical})\cr include the continuity correction. For further information, see for example
\code{\link[stats:prop.test]{stats::prop.test()}}.}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{Number of responses}
\item{N}{Total number}
\item{estimate}{The point estimate of the proportion}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{weights}{Weights of each strata, will be the same as the input unless
unspecified, then it will be the dynamically calculated weights.}
\item{method}{Type of method used}
}
\description{
Calculates the stratified Wilson confidence
interval for unequal proportions as described in
Xin YA, Su XG. Stratified Wilson and Newcombe confidence intervals
for multiple binomial proportions. \emph{Statistics in Biopharmaceutical Research}. 2010;2(3).
}
\details{
\deqn{\frac{\hat{p}_j + \frac{z^2_{\alpha/2}}{2n_j} \pm
z_{\alpha/2} \sqrt{\frac{\hat{p}_j(1 - \hat{p}_j)}{n_j} +
\frac{z^2_{\alpha/2}}{4n_j^2}}}{1 + \frac{z^2_{\alpha/2}}{n_j}}}
}
\examples{
# Stratified Wilson confidence interval with unequal probabilities

set.seed(1)
rsp <- sample(c(TRUE, FALSE), 100, TRUE)
strata_data <- data.frame(
  "f1" = sample(c("a", "b"), 100, TRUE),
  "f2" = sample(c("x", "y", "z"), 100, TRUE),
  stringsAsFactors = TRUE
)
strata <- interaction(strata_data)
n_strata <- ncol(table(rsp, strata)) # Number of strata

ci_prop_wilson_strata(
  x = rsp, strata = strata,
  conf.level = 0.90
)

# Not automatic setting of weights
ci_prop_wilson_strata(
  x = rsp, strata = strata,
  weights = rep(1 / n_strata, n_strata),
  conf.level = 0.90
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop_ci_diff.R
\name{ci_prop_diff_ha}
\alias{ci_prop_diff_ha}
\title{Anderson-Hauck Confidence Interval for Difference in Proportions}
\usage{
ci_prop_diff_ha(x, by, conf.level = 0.95, data = NULL)
}
\arguments{
\item{x}{(\code{binary}/\code{numeric}/\code{logical})\cr
vector of a binary values, i.e. a logical vector, or numeric with values \code{c(0, 1)}}

\item{by}{(\code{string}) \cr A character or factor vector with exactly two unique levels
identifying the two groups to compare. Can also be a column name if a data
frame provided in the \code{data} argument.}

\item{conf.level}{(\verb{scalar numeric}) \cr a scalar in (0,1) indicating the confidence level. Default is 0.95}

\item{data}{(\code{data.frame}) \cr Optional data frame containing the variables specified in \code{x} and \code{by}.}
}
\value{
An object containing the following components:

\item{n}{The number of responses for each group}
\item{N}{The total number in each group}
\item{estimate}{The point estimate of the difference in proportions}
\item{conf.low}{Lower bound of the confidence interval}
\item{conf.high}{Upper bound of the confidence interval}
\item{conf.level}{The confidence level used}
\item{method}{Anderson-Hauck Confidence Interval}
}
\description{
Anderson-Hauck Confidence Interval for Difference in Proportions
}
\details{
The confidence interval is given by:

\deqn{(\hat{p}_1 - \hat{p}_2) \pm \left[ \frac{1}{2 \min(n_1, n_2)} + z \sqrt{ \frac{\hat{p}_1 (1 - \hat{p}_1)}{n_1 - 1} + \frac{\hat{p}_2 (1 - \hat{p}_2)}{n_2 - 1} } \right]}.
}
\examples{
responses <- expand(c(9, 3), c(10, 10))
arm <- rep(c("treat", "control"), times = c(10, 10))

# Calculate 95\% confidence interval for difference in proportions
ci_prop_diff_ha(x = responses, by = arm)
}
\references{
Hauck WW, Anderson S. (1986) A comparison of large-sample confidence interval methods for the difference of two binomial probabilities The American Statistician 40(4). p.318-322.
\href{https://www.lexjansen.com/wuss/2016/127_Final_Paper_PDF.pdf}{Constructing Confidence Intervals for the Differences of Binomial Proportions in SAS}
}
